############################################################################
#
# Author:  Ruth Huey
#
# Copyright: M. Sanner TSRI 2004
#
#############################################################################

#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/atomTypeTools.py,v 1.23 2010/09/22 21:44:17 rhuey Exp $
#
#

"""
This module implements classes to support AutoDock atomtypes
    including:
        NonpolarHydrogenMerger: a class which detects and then
            merges nonpolarHydrogens with the heavy atoms to 
            which each is bound.
        LonepairMerger: a class which detects and then
            merges lonepairs with the heavy atoms to 
            which each is bound.
        AromaticCarbonManager: a class used for managing 
            names + autodock_element fields of
            aromatic carbons in cycles which it judges
            flat enough according to its 'cutoff' parameter
            to be considered aromatic.
        SolvationParameterizer: class used to assign solvation 
            parameters 'AtSolPar' and 'AtVol' to each atom in 
            receptor molecules for autodock305 experiment

"""

import os
from MolKit.molecule import AtomSet, BondSet
from MolKit.bondSelector import AromaticCycleBondSelector2
from AutoDockTools.sol_par import solvs
from PyBabel.atomTypes import AtomHybridization




class NonpolarHydrogenMerger:
    """
    removes non-polarhydrogens from an atomset;
    WARNING: builds bonds if none have been built previously
"""


    def mergeNPHS(self, atoms, renumber=1):
        if len(atoms.bonds[0])==0:
            print "WARNING atoms have no bonds....BUILDING THEM!!!"
            tops = atoms.top.uniq()
            for t in tops: t.buildBondsByDistance()
        #MAKE sure there are some hydrogens
        hs = atoms.get(lambda x: x.element=='H')
        if hs is None or len(hs)==0:
            return 0
        #Check whether there are any hydrogens with no bonds
        no_bnd_hs = hs.get(lambda x: len(x.bonds)==0)
        if len(no_bnd_hs):
            print "Warning: hydrogens, ", no_bnd_hs.name, " , with no bonds!"
        if len(no_bnd_hs)==len(hs):
            return 0
        bonded_hs = hs.get(lambda x: len(x.bonds)>0)
        #next check is superfluous
        if bonded_hs is None or len(bonded_hs)==0:
            print "Warning: no hydrogens with bonds!"
            return 0
        #Check whether there are any nphs hydrogens
        nphs = bonded_hs.get(lambda x: x.bonds[0].atom1.element=='C' \
                         or x.bonds[0].atom2.element=='C')
        if nphs is None or len(nphs)==0: 
            return 0 #if there aren't any, just return 0
        #if there are nphs, merge the charges + remove nphs
        tops = nphs.top.uniq() 
        for t in tops:
            #t_nphs are the nonpolarHydrogens in this molecule, 't'
            t_nphs = nphs.get(lambda x: x.top==t)

            #clean up t.allAtoms shortcut
            t.allAtoms = t.allAtoms - t_nphs

            #deal with the charges, if there are any
            chList = t_nphs[0]._charges.keys()
            for h in t_nphs:
                chs = h._charges.keys() 
                for c in chList:
                    if c not in chs:
                        chList.remove(c)
            if not len(chList):
                print 'no charges on carbons to increment'
            else:
                for chargeSet in chList:
                    for h in t_nphs:
                        if len(h.bonds)==0:
                            print h.full_name(), ' has no bonds!'
                        else:
                            c_atom = h.bonds[0].atom1
                            if c_atom==h: 
                                c_atom = h.bonds[0].atom2
                            c_atom._charges[chargeSet] = c_atom.charge + h.charge

            #have to do this loop separately because there may not be charges
            len_nphs = len(nphs)
            for h in nphs:
                #b = at.bonds[0]
                #could have hydrogens with >1bond!!!(YIKES)
                for b in h.bonds:
                    c = b.atom1
                    if c==h:
                        c = b.atom2
                    c.bonds.remove(b)
                h.bonds = BondSet()
                h.parent.remove(h)
                del h 
            if renumber:
                lenAts = len(t.chains.residues.atoms)
                assert lenAts==len(t.allAtoms)
                t.allAtoms.number = range(1, lenAts+1) 
        return len_nphs



class LonepairMerger:
    """
    removes lone pair 'atoms' from an atomset;
    WARNING: builds bonds if none have been built previously
This module implements a class which merges lone pair 'atoms' with the heavy
atom to which each is bound.
"""


    def mergeLPS(self, atoms, renumber=1):
        if len(atoms.bonds[0])==0:
            print "WARNING atoms have no bonds....BUILDING THEM!!!"
            tops = atoms.top.uniq()
            for t in tops: t.buildBondsByDistance()
        #lps = atoms.get(lambda x: x.element=='Xx' and \
                        #(x.name[0]=='L' or x.name[1]=='L'))
        lps = atoms.get(lambda x: x.element=='Lp' or x.element=='lp' or \
                        (x.element=='Xx' and (x.name[0]=='L' or x.name[1]=='L')))
        if lps is None or len(lps)==0: 
            return [] #if there aren't any, just return empty list
        tops = lps.top.uniq() 
        for mol in tops:
            #t_lps are the nonpolarHydrogens in this molecule, 'mol'
            mol_lps = lps.get(lambda x: x.top==mol)

            #clean up t.allAtoms shortcut
            mol.allAtoms = mol.allAtoms - mol_lps

            #deal with the charges, if there are any
            chList = mol_lps[0]._charges.keys()
            for lp in mol_lps:
                chs = lp._charges.keys() 
                for c in chList:
                    if c not in chs:
                        chList.remove(c)
            if not len(chList):
                print 'no charges on carbons to increment'
            else:
                for chargeSet in chList:
                    for lp in mol_lps:
                        if len(lp.bonds)==0:
                            print lp.full_name(), ' has no bonds!'
                        else:
                            c_atom = lp.bonds[0].atom1
                            if c_atom==lp: 
                                c_atom = lp.bonds[0].atom2
                            c_atom._charges[chargeSet] = c_atom.charge + lp.charge

            #have to do this loop separately because there may not be charges
            len_lps = len(lps)
            for lp in lps:
                #b = at.bonds[0]
                #could have hydrogens with >1bond!!!(YIKES)
                for b in lp.bonds:
                    c = b.atom1
                    if c==lp:
                        c = b.atom2
                    c.bonds.remove(b)
                lp.bonds = BondSet()
                lp.parent.remove(lp)
                del lp 
            if renumber:
                lenAts = len(mol.chains.residues.atoms)
                assert lenAts==len(mol.allAtoms)
                mol.allAtoms.number = range(1, lenAts+1) 
        return len_lps
            


class AromaticCarbonManager:
    """
    Used on ligands for managing names + autodock_element fields of
    aromatic carbons in cycles
    The parameter 'rename' determines whether aromatic carbons are
    named starting with 'A': 'rename' is True for AutoDock3 ligands 
    and False for AutoDock4 ligands whose written files have added 'type' field.
    """

    def __init__(self, cutoff=7.5, rename=True):
        self.cutoff = 7.5
        self.aromBndSel = AromaticCycleBondSelector2()
        self.rename = rename


    def setAromaticCarbons(self, molecule, cutoff=None, debug=False):
        assert len(molecule.allAtoms.bonds[0])
        if cutoff: 
            if cutoff!=self.cutoff:
                old_aromCs = molecule.allAtoms.get(lambda x:\
                                (x.element=='C' and x.autodock_element=='A'))
                if old_aromCs is not None and len(old_aromCs)!=0:
                    if debug: print "resetting ", len(old_aromCs), " prior aromCs"
                    self.set_carbon_names(old_aromCs, 'C')
            self.cutoff = cutoff
        typed_atoms = molecule.allAtoms.get(lambda x: hasattr(x, 'autodock_element'))
        currentAromCs = AtomSet()
        if typed_atoms is not None and len(typed_atoms)==len(molecule.allAtoms):
            currentAromCs = molecule.allAtoms.get(lambda x: (x.element=='C' and x.autodock_element=='A'))
        if debug:
            if currentAromCs is not None and len(currentAromCs):
                print "now: ", len(currentAromCs)
            else:
                print "now: no aromCs"
        aromBnds = self.aromBndSel.select(molecule.allAtoms.bonds[0],
                                                  self.cutoff)
        aromBndAts = self.aromBndSel.getAtoms(aromBnds)
        result = AtomSet()
        changed = AtomSet()
        if len(aromBndAts):
            aromCs =  AtomSet(filter(lambda x: x.element=='C', \
                                              aromBndAts))           
            aromCs = aromCs.uniq()
            if len(aromCs)>len(result):
                result = aromCs
            if debug: 
                print "len(aromCs)=", len(aromCs)
            changed = self.set_carbon_names(aromCs, 'A')
        #if len(changed)>len(result):
        #    result = changed
        #return  result
        return changed


    def change_aromatic_criteria(self, cutoff):
        """
        cutoff: angle between adjacent normals
        reset Aromatic carbons and redetect with
        new cutoff value. Cutoff is maximum possible value for 
        angle between normals to adjacent atoms in a cycle
        for carbons to considered aromatic. All normals for a
        single cycle must pass for any of its carbons to be
        considered aromatic.  This is a measure of the flatness
        of the cycle.
        """
        self.cutoff = cutoff
        return 


    def set_carbon_names(self, atoms, type):
        #set carbon names explicitly
        if not atoms or not len(atoms):
            return "ERROR: set_carbon_names called with no atoms"
        assert type in ['C','A']
        if not hasattr(atoms, 'autodock_element'):
            atoms.autodock_element = atoms.element
        changed = AtomSet()
        for at in atoms:
            if at.element!='C': continue
            if at.autodock_element!=type:
                if self.rename:
                    if len(at.name)>1:
                        at.name = type + at.name[1:]
                    else:
                        at.name = type
                at.autodock_element = type
                changed.append(at)
        return changed




class SolvationParameterizer:
    """
    adds AtSolPar and AtVol to standard protein atoms
"""
    def __init__(self, bblist=['C','O','N','CA'], 
                       nuc_acid_list= ['  A','  C','  G']):
        self.bblist = bblist
        self.nuc_acid_list = nuc_acid_list
        self.solvsKeys = solvs.keys()


    def addParameters(self, atoms):
        notfound = []
        for at in atoms:
            #use shortcut for peptide backbone atoms
            if at.name in self.bblist:
                atKey = at.name + '---'
            elif (at.name=='C2*' or at.name=='C2\'') \
                    and at.parent.type in self.nuc_acid_list:
                #special treatment for variants of nucleic acid names
                childnames = at.parent.atoms.name
                if 'O2*' in childnames or 'O2\'' in childnames:
                    #treat rna variant even more specially
                    atKey = at.name + ' R' + at.parent.type[2]
                    print 'special rna key', atKey
                else:
                    atKey = at.name + at.parent.type
            else:
                atKey = at.name + at.parent.type

            #if there are solvation parameters for this atom, add them
            if atKey in self.solvsKeys:
                at.AtVol, at.AtSolPar = solvs[atKey]
            else:
                at.AtVol, at.AtSolPar = (0.00, 0.00)
                if at.element!='H':
                    notfound.append(at)
        return notfound


class AutoDock4_AtomTyper:
    """
    sets autodock_element according to AutoDock4 criteria
"""

    def __init__(self, renameAtoms=0, verbose=False, set_aromatic_carbons=True):
        self.renameAtoms = 0 # whether to change first 
                             # char of carbon atom names to 'A' for AD4 type
        self.set_aromatic_carbons = set_aromatic_carbons
        if set_aromatic_carbons:
            #aromatic_carbon_manager:  
            # disable for a huge protein by setting typeAtoms=0
            self.acm = AromaticCarbonManager(rename=False)
        self.verbose = verbose
        self.acceptorList = ['NA','OA','SA']
        self.pep_aromList  = [
             'PHE_CD1', 'PHE_CG', 'PHE_CD2', 'PHE_CE1', 'PHE_CE2', 'PHE_CZ', 
             'PHE_AD1', 'PHE_AG', 'PHE_AD2', 'PHE_AE1', 'PHE_AE2', 'PHE_AZ', 
             'TYR_CD1', 'TYR_CG', 'TYR_CD2', 'TYR_CE1', 'TYR_CE2', 'TYR_CZ', 
             'TYR_AD1', 'TYR_AG', 'TYR_AD2', 'TYR_AE1', 'TYR_AE2', 'TYR_AZ', 
             'TRP_CG', 'TRP_CD1','TRP_CD2', 'TRP_CE2', 'TRP_CE3','TRP_CZ2', 'TRP_CH2', 'TRP_CZ3',
             'TRP_AG', 'TRP_AD1','TRP_AD2', 'TRP_AE2', 'TRP_AE3','TRP_AZ2', 'TRP_AH2', 'TRP_AZ3',
             'HIS_AD2', 'HIS_AE1', 'HIS_AG', 
             'HIS_CD2', 'HIS_CE1', 'HIS_CG'] 
 
     
    def setAutoDockElements(self, mol, typeAtoms=0, reassign=False, splitAcceptors=False):
        if os.path.splitext(mol.parser.filename)[-1]=='.pdbqt' and reassign is False:
            if self.verbose:
                print 'setAutoDockElements unnecessary:\n', mol.name, ' already has AD4 atomtypes: not reassigned!'
            return
        #check that the types have already been set, first
        if self.set_aromatic_carbons:
            #this sets the autodock_element for the planar cyclic carbons to 'A'
            aromatic_carbons = self.acm.setAromaticCarbons(mol)
        d = {}
        ah = AtomHybridization()
        if not len(mol.allAtoms.bonds[0]):
            mol.buildBondsByDistance()
        ats_with_babel_type = filter(lambda x: hasattr(x, 'babel_type'), mol.allAtoms)
        if typeAtoms or len(ats_with_babel_type)!=len(mol.allAtoms):
            if self.verbose:
                print "assigning babel_types"
            ah = AtomHybridization()
            ah.assignHybridization(mol.allAtoms)
        for item in mol.allAtoms:
            if not hasattr(item, 'autodock_element'):
                #item.element = item.element.upper()
                item.autodock_element = item.element
            if item.element=='H':
                if len(item.bonds):
                    n = item.bonds[0].neighborAtom(item)
                    if n.element not in ['C','A']:
                        item.autodock_element = 'HD'
                else:
                    #note: if there are no bonds, assume HD 
                    item.autodock_element = 'HD'
            elif item.element=='N':
                #this has already been done at beginning of for loop
                #if item.babel_type in ['N3+','Ntr', 'Nox']:
                #    item.autodock_element = 'N'
                if item.babel_type in ['N3','N2','N1']:
                    item.autodock_element = item.element + 'A'
                elif item.babel_type in ['Nam', 'Npl', 'Ng+']:
                    if len(item.bonds)==2:
                        item.autodock_element = item.element + 'A'
            elif item.element=='O':
                item.autodock_element = item.element + 'A'
            elif item.element=='S':
                #possible sulphur babel_types->['S3+','S3','S2','Sac','Sox','S']
                if item.babel_type not in ['Sox', 'Sac']:
                    item.autodock_element = item.element + 'A'
            elif item.element=='C':
                if item.parent.type+'_' + item.name in self.pep_aromList:
                    item.autodock_element = 'A'
                    if self.renameAtoms:  # 
                        if len(item.name)==1:
                            item.name = 'A'
                        else:
                            item.name = 'A' + item.name[1:]
                elif hasattr(item, 'autodock_element'):
                    item.autodock_element = item.autodock_element
            if splitAcceptors and item.autodock_element in self.acceptorList:
                #if item has a bond to a hydrogen:
                #   change acceptor type 'XA' to 'XB' for 'B'oth 'A'cceptor and 'D'onor 
                if 'H' in item.bonds.getAtoms().element:
                    #O->OB;N->NB;S->SB
                    item.autodock_element = item.element + 'B'
            d[item.autodock_element] = 1
        type_list = d.keys()
        type_list.sort()
        mol.types = type_list


if __name__ == "__main__":
    from MolKit import Read
    m = Read("/mgl/work4/rhuey/dev23/hsg1_no_conects.pdbq")[0]
    solP = SolvationParameterizer()
    ll = solP.addParameters(m.allAtoms)
    #NB: hydrogens get 0.0 0.0
    print "len(notfound) = ", len(ll)
    for i in ll: 
        print i.full_name(),
    from MolKit.pdbWriter import PdbqsWriter
    writer = PdbqsWriter()
    writer.write("/mgl/work4/rhuey/dev23/test_hsg1.pdbqs", m.allAtoms)




#!/usr/bin/env python

import os
from string import find, strip

from MolKit import Read
from MolKit.pdbWriter import PdbqtWriter
from AutoDockTools.atomTypeTools import AutoDock4_AtomTyper
from AutoDockTools.GridParameters import GridParameters, grid_parameter_list




if __name__ == '__main__':
    import sys
    import getopt


    def usage():
        "Print helpful, accurate usage statement to stdout."
        print "Usage: gpf4_to_gpf3.py -s gpf4_stem -r receptor_filename -l ligand_filename"
        print
        print "    Description of command..."
        print "        [-s]    gpf4_stem"
        print "        [-r]    receptor_filename"
        print "        [-l]    ligand_filename"
        print "    Optional parameters:"
        print "        [-o]    outputfilename"
        print "        [-v]    verbose output"


    # process command arguments
    try:
        #opt_list, args = getopt.getopt(sys.argv[1:], 'r:o:bs:ampvHG')
        opt_list, args = getopt.getopt(sys.argv[1:], 's:r:l:o:v')

    except getopt.GetoptError, msg:
        print 'gpf4_to_gpf3.py: %s' %msg
        usage()
        sys.exit(2)

    # initialize required parameters
    #-s: pdbq_stem
    gpf4_filename =  None

    #-r: receptor_filename
    receptor_filename =  None

    #-l: ligand_filename
    ligand_filename =  None

    # optional parameters
    # optional parameters
    verbose = None
    outputfilename = None

    #'s:r:l:v'
    for o, a in opt_list:
        if o in ('-s', '--s'):
            gpf4_stem = a
            gpf4_filename = a + '.gpf'
            if verbose: print 'set gpf4_filename to ', gpf4_filename
        if o in ('-r', '--r'):
            receptor_filename = a
            if verbose: print 'set receptor_filename to ', receptor_filename
        if o in ('-l', '--l'):
            ligand_filename = a
            if verbose: print 'set ligand_filename to ', ligand_filename
        if o in ('-o', '--o'):
            outputfilename = a
            if verbose: print 'set outputfilename to ', outputfilename
        if o in ('-v', '--v'):
            verbose = True
            if verbose: print 'set verbose to ', True
        if o in ('-h', '--'):
            usage()
            sys.exit()


    if not ligand_filename:
        print 'gpf4_to_gpf3: ligand_filename must be specified.'
        usage()
        sys.exit()
    if not receptor_filename:
        receptor_filename = gpf4_stem + '.pdbqs'
        print "gpf4_to_gpf3: using default receptor_filename:", receptor_filename
    if not gpf4_filename:
        print 'gpf4_to_gpf3: no gpf4_filename specified using GridParameter defaults'

    #what about nucleic acids???
    GPO3 = GridParameters()
    GPO3.read4(gpf4_filename)
    GPO3.read(gpf4_filename)
    if verbose: print 'read ', gpf4_filename
    #have to set the ligand_types
    GPO3.set_ligand(ligand_filename)
    #build an autogrid3 'types' string from autogrid4 ligand_types
    GPO3.set_ligand_types3(GPO3['ligand_types']['value'])
    if verbose: print 'set ligand to  ', ligand_filename
    GPO3.set_receptor(receptor_filename)
    if verbose: print 'set receptor to  ', receptor_filename
    if outputfilename is None:
        outputfilename = gpf4_stem + '_3.gpf'
    GPO3.write(outputfilename, grid_parameter_list)
    if verbose: 
        print "wrote ", outputfilename, ' using:'
        for p in grid_parameter_list:
            print p
    

# To execute this command type:
# gpf4_to_gpf3.py -s gpf4_stem -v







#
#
#
#
# $Id: test_prepare_gpf4.py,v 1.5 2014/03/24 21:16:25 rhuey Exp $
#

import unittest, os, sys
from MolKit import Read
from AutoDockTools.GridParameters import GridParameters

class BaseTests(unittest.TestCase):

    def test_default(self):
        """
    prepare_gpf4.py -l pdbqt_file -r pdbqt_file -"
       [-l ligand_filename]"
       [-r receptor_filename]"
   Optional parameters:"
       [-i reference_gpf_filename]"
       [-o output_gpf_filename]"
       [-p parameter=newvalue]"
       [-v]"

        """
        filename = "test_default4.gpf"
        if os.path.exists(filename):
            cmd = "rm  " + filename
            os.system(cmd)
        cmd = 'set PYTHONPATH=%s'%sys.path[0]
        cmd = cmd + "; ../prepare_gpf4.py -l 1ebg_lig.pdbqt -r 1ebg_rec.pdbqt -o " + filename 
        os.system(cmd)
        self.assertEqual(os.path.exists(filename), 1)

    def test_set_ligand_types_directly_gpf(self):
        """
        test setting ligand_types parameters using list instead of ligand
        options
           -p ligand_types=
        """
        filename = "test_set_ligand_types_directly.gpf"
        if os.path.exists(filename):
            cmd = "rm  " + filename
            os.system(cmd)
        cmd = 'set PYTHONPATH=%s'%sys.path[0]
        cmd = cmd + "; ../prepare_gpf4.py -p ligand_types=P,C,OA,HD,N  -r 1ebg_rec.pdbqt -i ref_1ebg.gpf -o " + filename
        os.system(cmd)
        ref_gpo = GridParameters()
        ref_gpo.read4('ref_1ebg.gpf')
        gpo = GridParameters()
        gpo.read4(filename)
        for key,refval in ref_gpo.items():
            if gpo[key]['value']!=refval['value']:
                print "failing.. ", key, '  newvalue=', gpo[key]['value']
                print '  refvalue=', ref_gpo[key]['value']
            self.assertEqual(gpo[key]['value'], refval['value'])

    def test_use_reference_gpf(self):
        """
        test setting parameters using reference gpf
        options
           -i gpffilename
        """
        filename = "test_ref4.gpf"
        if os.path.exists(filename):
            cmd = "rm  " + filename
            os.system(cmd)
        cmd = 'set PYTHONPATH=%s'%sys.path[0]
        cmd = cmd + "; ../prepare_gpf4.py -l 1ebg_lig.pdbqt -r 1ebg_rec.pdbqt -i ref_1ebg.gpf -o " + filename
        os.system(cmd)
        ref_gpo = GridParameters()
        ref_gpo.read4('ref_1ebg.gpf')
        gpo = GridParameters()
        gpo.read4(filename)
        for key,refval in ref_gpo.items():
            if gpo[key]['value']!=refval['value']:
                print "failing.. ", key, '  newvalue=', gpo[key]['value']
                print '  refvalue=', ref_gpo[key]['value']
            self.assertEqual(gpo[key]['value'], refval['value'])


    def test_setting_parameter(self):
        """
        test setting parameter directly
        options
           -p parameter=newvalue
        """
        filename = "test_param4.gpf"
        if os.path.exists(filename):
            cmd = "rm  " + filename
            os.system(cmd)
        cmd = 'set PYTHONPATH=%s'%sys.path[0]
        cmd = cmd + "; ../prepare_gpf4.py -l 1ebg_lig.pdbqt -r 1ebg_rec.pdbqt -p spacing=1.0  -o " + filename
        os.system(cmd)
        gpo = GridParameters()
        gpo.read4(filename)
        self.assertEqual(gpo['spacing']['value'], 1.0) 


    def test_setting_2_parameters(self):
        """
        test setting 2 parameters directly
        options
           -p parameter=newvalue
           -p parameter2=newvalue2
        """
        filename = "test_2params4.gpf"
        if os.path.exists(filename):
            cmd = "rm  " + filename
            os.system(cmd)
        cmd = 'set PYTHONPATH=%s'%sys.path[0]
        cmd = cmd + "; ../prepare_gpf4.py -l 1ebg_lig.pdbqt -r 1ebg_rec.pdbqt -p spacing=1.0 -p gridcenter='1.0 2.0 3.0' -o " + filename
        os.system(cmd)
        gpo = GridParameters()
        gpo.read4(filename)
        self.assertEqual(gpo['spacing']['value'], 1.0) 
        self.assertEqual(int(gpo['gridcenter']['value'][0]), 1.0) 


    def test_setting_parameter_file_parameter(self):
        """
        test setting parameter_file directly
        options
           -p parameter_file='ad4_local.dat'
        """
        filename = "test_setting_parameter_file.gpf"
        if os.path.exists(filename):
            cmd = "rm  " + filename
            os.system(cmd)
        cmd = 'set PYTHONPATH=%s'%sys.path[0]
        cmd = cmd + "; ../prepare_gpf4.py -l 1ebg_lig.pdbqt -r 1ebg_rec.pdbqt -p 'parameter_file=ad4_local.dat' -o " + filename
        os.system(cmd)
        gpo = GridParameters()
        gpo.read4(filename)
        self.assertEqual(gpo['custom_parameter_file']['value'], 1.0) 
        self.assertEqual(gpo['parameter_file']['value'], 'ad4_local.dat') 

    def test_center_on_ligand(self):
        """
        test center on ligand
        options
           -y center_on_ligand=True
        """
        filename = "test_center_on_ligand.gpf"
        if os.path.exists(filename):
            cmd = "rm  " + filename
            os.system(cmd)
        cmd = 'set PYTHONPATH=%s'%sys.path[0]
        cmd = cmd + "; ../prepare_gpf4.py -l 1ebg_lig.pdbqt -r 1ebg_rec.pdbqt -y -o " + filename
        os.system(cmd)
        gpo = GridParameters()
        gpo.read4(filename)
        self.assertEqual(gpo['gridcenter']['value'],  [17.646, 5.470, 20.124])
        fptr = open("test_center_on_ligand.gpf")
        lines = fptr.readlines()
        fptr.close()
        gridcenter_line = lines[6].strip()
        self.assertEqual(gridcenter_line, "gridcenter 17.646 5.470 20.124       # xyz-coordinates or auto")



if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#
# 
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/Utilities24/prepare_ligand4.py,v 1.10 2010/07/31 00:14:13 rhuey Exp $
#
import os 

from MolKit import Read

from AutoDockTools.MoleculePreparation import AD4LigandPreparation



if __name__ == '__main__':
    import sys
    import getopt


    def usage():
        "Print helpful, accurate usage statement to stdout."
        print "Usage: prepare_ligand4.py -l filename"
        print
        print "    Description of command..."
        print "         -l     ligand_filename (.pdb or .mol2 or .pdbq format)"
        print "    Optional parameters:"
        print "        [-v]    verbose output"
        print "        [-o pdbqt_filename] (default output filename is ligand_filename_stem + .pdbqt)"
        print "        [-d]    dictionary to write types list and number of active torsions "

        print "        [-A]    type(s) of repairs to make:\n\t\t bonds_hydrogens, bonds, hydrogens (default is to do no repairs)"
        print "        [-C]    do not add charges (default is to add gasteiger charges)"
        print "        [-p]    preserve input charges on atom type, eg -p Zn"
        print "               (default is not to preserve charges on any specific atom type)"
        print "        [-U]    cleanup type:\n\t\t nphs_lps, nphs, lps, '' (default is 'nphs_lps') "
        print "        [-B]    type(s) of bonds to allow to rotate "
        print "               (default sets 'backbone' rotatable and 'amide' + 'guanidinium' non-rotatable)"
        print "        [-R]    index for root"
        print "        [-F]    check for and use largest non-bonded fragment (default is not to do this)"
        print "        [-M]    interactive (default is automatic output)"
        print "        [-I]    string of bonds to inactivate composed of "
        print "                   of zero-based atom indices eg 5_13_2_10  "
        print "                   will inactivate atoms[5]-atoms[13] bond "
        print "                               and atoms[2]-atoms[10] bond "
        print "                      (default is not to inactivate any specific bonds)"
        print "        [-Z]    inactivate all active torsions     "
        print "                      (default is leave all rotatable active except amide and guanidinium)"
        print "        [-g]    attach all nonbonded fragments "
        print "        [-s]    attach all nonbonded singletons: "
        print "                   NB: sets attach all nonbonded fragments too"
        print "                      (default is not to do this)"


    # process command arguments
    try:
        opt_list, args = getopt.getopt(sys.argv[1:], 'l:vo:d:A:Cp:U:B:R:MFI:Zgsh')
    except getopt.GetoptError, msg:
        print 'prepare_ligand4.py: %s' %msg
        usage()
        sys.exit(2)

    # initialize required parameters
    #-l: ligand
    ligand_filename =  None
    # optional parameters
    verbose = None
    add_bonds = False
    #-A: repairs to make: add bonds and/or hydrogens
    repairs = ""
    #-C  default: add gasteiger charges 
    charges_to_add = 'gasteiger'
    #-p preserve charges on specific atom types
    preserve_charge_types=''
    #-U: cleanup by merging nphs_lps, nphs, lps
    cleanup  = "nphs_lps"
    #-B named rotatable bond type(s) to allow to rotate
    #allowed_bonds = ""
    allowed_bonds = "backbone"
    #-r  root
    root = 'auto'
    #-o outputfilename
    outputfilename = None
    #-F check_for_fragments
    check_for_fragments = False
    #-I bonds_to_inactivate
    bonds_to_inactivate = ""
    #-Z inactivate_all_torsions
    inactivate_all_torsions = False
    #-g attach_nonbonded_fragments
    attach_nonbonded_fragments = False
    #-s attach_nonbonded_singletons
    attach_singletons = False
    #-m mode 
    mode = 'automatic'
    #-d dictionary
    dict = None

    #'l:vo:d:A:CKU:B:R:MFI:Zgs'
    for o, a in opt_list:
        #print "o=", o, " a=", a
        if o in ('-l', '--l'):
            ligand_filename = a
            if verbose: print 'set ligand_filename to ', a
        if o in ('-v', '--v'):
            verbose = True
            if verbose: print 'set verbose to ', True
        if o in ('-o', '--o'):
            outputfilename = a
            if verbose: print 'set outputfilename to ', a
        if o in ('-d', '--d'):
            dict = a
            if verbose: print 'set dict to ', a
        if o in ('-A', '--A'):
            repairs = a
            if verbose: print 'set repairs to ', a
        if o in ('-C', '--C'):
            charges_to_add = None
            if verbose: print 'do not add charges'
        if o in ('-p', '--p'):
            preserve_charge_types+=a
            preserve_charge_types+=','
            if verbose: print 'preserve initial charges on ', preserve_charge_types
        if o in ('-U', '--U'):
            cleanup  = a
            if verbose: print 'set cleanup to merge ', a
        if o in ('-B', '--B'):
            allowed_bonds = a
            if verbose: print 'allow ', a, 'bonds set to rotate'
        if o in ('-R', '--R'):
            root = a
            if verbose: print 'set root to ', root
        if o in ('-F', '--F'):
            check_for_fragments = True
            if verbose: print 'set check_for_fragments to True'
        if o in ('-M', '--M'):
            mode = a
            if verbose: print 'set mode to ', a
        if o in ('-I', '--I'):
            bonds_to_inactivate = a
            if verbose: print 'set bonds_to_inactivate to ', a
        if o in ('-Z', '--Z'):
            inactivate_all_torsions = True
            if verbose: print 'set inactivate_all_torsions to ', inactivate_all_torsions
        if o in ('-g', '--g'):
            attach_nonbonded_fragments = True
            if verbose: print 'set attach_nonbonded_fragments to ', attach_nonbonded_fragments
        if o in ('-s', '--s'):
            attach_singletons = True
            if verbose: print 'set attach_singletons to ', attach_singletons
        if o in ('-h', '--'):
            usage()
            sys.exit()


    if not  ligand_filename:
        print 'prepare_ligand4: ligand filename must be specified.'
        usage()
        sys.exit()

    if attach_singletons:
        attach_nonbonded_fragments = True
        if verbose: print "using attach_singletons so attach_nonbonded_fragments also"
    
    mols = Read(ligand_filename)
    if verbose: print 'read ', ligand_filename
    mol = mols[0]
    if len(mols)>1:
        if verbose: 
            print "more than one molecule in file"
        #use the one molecule with the most atoms
        ctr = 1
        for m in mols[1:]:
            ctr += 1
            if len(m.allAtoms)>len(mol.allAtoms):
                mol = m
                if verbose:
                    print "mol set to ", ctr, "th molecule with", len(mol.allAtoms), "atoms"
    coord_dict = {}
    for a in mol.allAtoms: coord_dict[a] = a.coords


    mol.buildBondsByDistance()
    if charges_to_add is not None:
        preserved = {}
        preserved_types = preserve_charge_types.split(',') 
        for t in preserved_types:
            if not len(t): continue
            ats = mol.allAtoms.get(lambda x: x.autodock_element==t)
            for a in ats:
                if a.chargeSet is not None:
                    preserved[a] = [a.chargeSet, a.charge]



    if verbose:
        print "setting up LPO with mode=", mode,
        print "and outputfilename= ", outputfilename
        print "and check_for_fragments=", check_for_fragments
        print "and bonds_to_inactivate=", bonds_to_inactivate
    LPO = AD4LigandPreparation(mol, mode, repairs, charges_to_add, 
                            cleanup, allowed_bonds, root, 
                            outputfilename=outputfilename,
                            dict=dict, check_for_fragments=check_for_fragments,
                            bonds_to_inactivate=bonds_to_inactivate, 
                            inactivate_all_torsions=inactivate_all_torsions,
                            attach_nonbonded_fragments=attach_nonbonded_fragments,
                            attach_singletons=attach_singletons)
    #do something about atoms with too many bonds (?)
    #FIX THIS: could be peptide ligand (???)
    #          ??use isPeptide to decide chargeSet??
    if charges_to_add is not None:
        #restore any previous charges
        for atom, chargeList in preserved.items():
            atom._charges[chargeList[0]] = chargeList[1]
            atom.chargeSet = chargeList[0]
    if verbose: print "returning ", mol.returnCode 
    bad_list = []
    for a in mol.allAtoms:
        if a in coord_dict.keys() and a.coords!=coord_dict[a]: 
            bad_list.append(a)
    if len(bad_list):
        print len(bad_list), ' atom coordinates changed!'    
        for a in bad_list:
            print a.name, ":", coord_dict[a], ' -> ', a.coords
    else:
        if verbose: print "No change in atomic coordinates"
    if mol.returnCode!=0: 
        sys.stderr.write(mol.returnMsg+"\n")
    sys.exit(mol.returnCode)


# To execute this command type:
# prepare_ligand4.py -l pdb_file -v







#!/usr/bin/env python
#
# 
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/Utilities24/prepare_configFile.py,v 1.1 2012/10/11 21:14:47 rhuey Exp $
# $Id: prepare_configFile.py,v 1.1 2012/10/11 21:14:47 rhuey Exp $
#

import string
import os.path
from MolKit import Read #?detect center of molecule+bounding box???
from AutoDockTools.DockingParameters import ConfigFileMaker

                

def usage():
    print "Usage: prepare_configFile.py -l pdbqt_file -r pdbqt_file"
    print "Description of command..."
    print "Prepare a config file for AutoDock vina: config.txt"
    print "  containing input parameters"
    print "    -l ligand_filename"
    print "    -r receptor_filename"
    print
    print "Optional parameters:"
    print "    [-a center_x]" #default [0,0,0]?
    print "    [-b center_y]"
    print "    [-c center_z]"
    print "    [-X size_x]"  #default is bounding box of receptor?
    print "    [-Y size_y]"
    print "    [-Z size_z]"
    print "    [-o output config filename (default is 'config.txt')]"
    print "    [-L optionally, write log file]"
    print "    [-i template config filename]"
    print "    [-x flex_filename]"
    print "    [-p parameter_name=new_value]"
    print "    [-S write score_only config file ]"
    print "    [-L write local_only config file ]"
    print "    [-R write randomize_only config file ]"
    print "    [-F filename for new config file ]"
    print "    [-v] verbose output"
    print

    
if __name__ == '__main__':
    import getopt
    import sys

    try:
        opt_list, args = getopt.getopt(sys.argv[1:], 'l:r:a:b:c:X:Y:Z:o:L:i:x:p:F:SLRFvh')
    except getopt.GetoptError, msg:
        print 'prepare_configFile.py: %s' % msg
        usage()
        sys.exit(2)

    receptor_filename = ligand_filename = None
    out = None
    reference_config = None
    flex_filename = None
    kw = {}
    verbose = None
    config_filename = ""
    kw = {}
    kw['parameters'] = []
    for o, a in opt_list:
        if verbose: print "o=", o, ' a=', a
        if o in ('-h', '--'):
            usage()
            sys.exit()
        if o in ('-v', '--v'):
            verbose = 1
            if verbose: print 'verbose output'
        if o in ('-r', '--r'):   #receptor filename
            receptor_filename = a
            if verbose: print 'receptor_filename =', receptor_filename
        if o in ('-x', '--x'):   #flex_filename 
            flex_filename = a
            kw['flex'] = a
            if verbose: print 'flex =', flex_filename
        if o in ('-l', '--l'):   #ligand filename
            ligand_filename = a
            if verbose: print 'ligand_filename =', ligand_filename
        if o in ('-a', '--a'):   #center_x 
            center_x = a
            kw['center_x'] = a
            if verbose: print 'center_x =', center_x
        if o in ('-b', '--b'):   #center_y 
            center_y = a
            kw['center_y'] = a
            if verbose: print 'center_y =', center_y
        if o in ('-c', '--c'):   #center_z 
            center_z = a
            kw['center_z'] = a
            if verbose: print 'center_z =', center_z
        if o in ('-X', '--X'):   #size_x 
            size_x = a
            kw['size_x'] = a
            if verbose: print 'size_x =', size_x
        if o in ('-Y', '--Y'):   #size_y 
            size_y = a
            kw['size_y'] = a
            if verbose: print 'size_y =', size_y
        if o in ('-Z', '--Z'):   #size_z 
            size_z = a
            kw['size_z'] = a
            if verbose: print 'size_z =', size_z
        #optional : 'l:r:a:b:c:X:Y:Z:o:i:x:p:SLRvh'
        if o in ('-o', '--o'):   #out filename (pdbqt)
            out = a
            kw['out'] = a
            if verbose: print 'out =', out
        if o in ('-L', '--L'):   #log filename
            log = a
            kw['log'] = a
            if verbose: print 'log =', log
        if o in ('-i', '--i'):   #reference_config filename
            reference_config = a
            if verbose: print 'reference_config =', reference_config
        if o in ('-p', '--p'):   #parameter
            kw['parameter'].append(a)
            if verbose: print 'kw =', kw
    #[-S write score_only config file ],[-L write local_only config file ],[-R write randomize_only config file ]
        if o in ('-S', '--S'):   #score_only
            score_only = 1
            #parameters.append((o,1))
            kw['score_only'] = True
            if verbose: print '1: kw =', kw
        if o in ('-L', '--L'):   #parameter_list_to_write
            kw['local_only'] = True
            local_search = 1
            if verbose: print '2: kw =', kw
        if o in ('-R', '--R'):   #randomize_only
            randomize_only = True
            kw['randomize_only'] = True
            if verbose: print '3: kw =', kw
        if o in ('-F', '--F'):   # config filename
            config_filename = a
            kw['config_filename'] = a
            if verbose: print '3: kw =', kw
        if o in ('-h', '--'):
            usage()
            sys.exit()

    if (not receptor_filename) or (not ligand_filename):
        print "prepare_configFile.py: ligand and receptor filenames"
        print "                    must be specified."
        usage()
        sys.exit()

    if len(config_filename)==0:
        config_filename = "config.txt"
    cm = ConfigFileMaker(receptor=receptor_filename,ligand=ligand_filename, **kw)
    if verbose: print "writing new config file:", config_filename
    cm.write(config_filename)
    if verbose: print "results will be written by vina in ", out

    
#prepare_configFile.py -l indinavir.pdbqt -r 1hsg.pdbqt 




#
#
#
#
# $Id: test_prepare_gpf.py,v 1.1 2005/07/21 22:47:02 sowjanya Exp $
#

import unittest, os, sys
from string import split, find
from MolKit import Read
from AutoDockTools.GridParameters import GridParameters

class BaseTests(unittest.TestCase):

    def test_default(self):
        """
    prepare_gpf.py -l pdbq_file -r pdbqs_file -"
       [-l ligand_filename]"
       [-r receptor_filename]"
   Optional parameters:"
       [-i reference_gpf_filename]"
       [-o output_gpf_filename]"
       [-p parameter=newvalue]"
       [-v]"

        """
        filename = "test_default.gpf"
        if os.path.exists(filename):
            cmd = "rm  " + filename
            os.system(cmd)
        cmd = 'set PYTHONPATH=%s'%sys.path[0]
        cmd = cmd + "; ../prepare_gpf.py -l ind.pdbq -r hsg1.pdbqs -o " + filename 
        os.system(cmd)
        self.assertEqual(os.path.exists(filename), 1)


    def test_use_reference_gpf(self):
        """
        test setting parameters using reference gpf
        options
           -i gpffilename
        """
        filename = "test_ref.gpf"
        if os.path.exists(filename):
            cmd = "rm  " + filename
            os.system(cmd)
        cmd = 'set PYTHONPATH=%s'%sys.path[0]
        cmd =  cmd + "; ../prepare_gpf.py -l ind.pdbq -r hsg1.pdbqs -i hsg1.gpf -o " + filename
        os.system(cmd)
        ref_gpo = GridParameters()
        ref_gpo.read('hsg1.gpf')
        gpo = GridParameters()
        gpo.read(filename)
        for key,refval in ref_gpo.items():
            if gpo[key]['value']!=refval['value']:
                print "failing.. ", key
            self.assertEqual(gpo[key]['value'], refval['value'])


    def test_setting_parameter(self):
        """
        test setting parameter directly
        options
           -p parameter=newvalue
        """
        filename = "test_param.gpf"
        if os.path.exists(filename):
            cmd = "rm  " + filename
            os.system(cmd)
        cmd = 'set PYTHONPATH=%s'%sys.path[0]
        cmd = cmd + "; ../prepare_gpf.py -l ind.pdbq -r hsg1.pdbqs -p spacing=1.0  -o " + filename
        os.system(cmd)
        gpo = GridParameters()
        gpo.read(filename)
        self.assertEqual(gpo['spacing']['value'], 1.0) 


    def test_setting_2_parameters(self):
        """
        test setting 2 parameters directly
        options
           -p parameter=newvalue
           -p parameter2=newvalue2
        """
        filename = "test_2params.gpf"
        if os.path.exists(filename):
            cmd = "rm  " + filename
            os.system(cmd)
        cmd = 'set PYTHONPATH=%s'%sys.path[0]
        cmd = cmd + "; ../prepare_gpf.py -l ind.pdbq -r hsg1.pdbqs -p spacing=1.0 -p gridcenter='1.0 2.0 3.0' -o " + filename
        os.system(cmd)
        gpo = GridParameters()
        gpo.read(filename)
        self.assertEqual(gpo['spacing']['value'], 1.0) 
        self.assertEqual(int(gpo['gridcenter']['value'][0]), 1.0) 


if __name__ == '__main__':
    unittest.main()



########################################################################
#
#    Vision Node - Python source code - file generated by vision
#    Tuesday 25 July 2006 11:40:00 
#    
#       The Scripps Research Institute (TSRI)
#       Molecular Graphics Lab
#       La Jolla, CA 92037, USA
#
# Copyright: Daniel Stoffler, Michel Sanner and TSRI
#   
# revision: Guillaume Vareille
#  
#########################################################################
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/VisionInterface/Docking.py,v 1.1 2006/07/25 23:35:30 rhuey Exp $
#
# $Id: Docking.py,v 1.1 2006/07/25 23:35:30 rhuey Exp $
#

# import node's base class node
from NetworkEditor.items import NetworkNode
class Docking(NetworkNode):
    mRequiredTypes = {}
    mRequiredSynonyms = [
    ]
    def __init__(self, constrkw = {},  name='Docking', **kw):
        kw['constrkw'] = constrkw
        kw['name'] = name
        apply( NetworkNode.__init__, (self,), kw)
        code = """def doit(self, docking_logs, rmsTool):
        from AutoDockTools.Docking import Docking
        d = Docking()
        for f in docking_logs:
                d.readDlg(f)
        d.clusterer.rmsTool = rmsTool
        self.outputData(docking = d)           
                    
        
## to ouput data on port docking use
## self.outputData(docking=data)


"""
        self.configure(function=code)
        self.inputPortsDescr.append(
            {'name': 'docking_logs', 'cast': True, 'datatype': 'list', 'balloon': 'list of docking log files to be read into as a single Docking', 'height': 12, 'width': 12, 'shape': 'oval', 'color': 'cyan'})
        self.inputPortsDescr.append(
            {'name': 'rmsTool', 'cast': True, 'datatype': 'None', 'balloon': 'rmsTool to be used for clustering', 'height': 8, 'width': 12, 'shape': 'diamond', 'color': 'white'})
        self.outputPortsDescr.append(
            {'name': 'docking', 'datatype': 'None', 'balloon': 'docking instance', 'height': 8, 'width': 12, 'shape': 'diamond', 'color': 'white'})


    def beforeAddingToNetwork(self, net):
        try:
            ed = net.getEditor()
        except:
            import traceback; traceback.print_exc()
            print 'Warning! Could not import widgets'




#!/usr/bin/env python
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/Utilities24/energy_average_maps.py,v 1.4 2009/03/23 16:40:44 rhuey Exp $
import glob, math

if __name__ == '__main__':
    import sys
    import getopt


    def usage():
        "Print helpful, accurate usage statement to stdout."
        print "Usage: energy_average_maps.py "
        print 
        print "    Invoke this script in directory containing all maps..."
        print "    Optional parameters:"
        print "        [-s]    map stem ('weighted')"
        print "        [-v]    verbose output"


    # process command arguments
    try:
        opt_list, args = getopt.getopt(sys.argv[1:], 's:vh')
    except getopt.GetoptError, msg:
        print 'energy_average_maps.py: %s' %msg
        usage()
        sys.exit(2)

    # initialize required parameters
    # initialize optional parameters
    #-s: map stem
    map_stem = 'weighted'
    verbose = False
    #'s:vh'
    for o, a in opt_list:
        #print "o=", o, " a=", a
        if o in ('-s', '--s'):
            map_stem = a
            if verbose: print 'set map_stem to ', a
        if o in ('-v', '--v'):
            verbose = True
            if verbose: print 'set verbose to ', True
        if o in ('-h', '--'):
            usage()
            sys.exit()


    rt = 0.592
    stems = {}
    atomtypes = {}

    all_maps = glob.glob("*.map")
    for m in all_maps:
        ll = m.split('.')
        stems[ll[0]] = 1
        atomtypes[ll[1]] = 1

    #remove 'e' and 'd' which are computed differently
    del(atomtypes['e'])
    del(atomtypes['d'])

    all_stems = stems.keys()
    all_stems.sort()
    if verbose: print 'all_stems=', all_stems

    all_types = atomtypes.keys()
    all_types.sort()
    if verbose: print 'all_types=', all_types

    all_lines = {}
    for t in all_types:      #loop over atom types
        for s in all_stems:  #loop over receptors
            all_lines[s] = {}
            fn = "%s.%s.map" %(s,t)
            if verbose: print "opening ", fn
            fptr = open(fn)
            all = fptr.readlines()
            fptr.close()
            #skip the first 6 lines of header and convert readlines output(strings) to floats
            all_lines[s][t] = map(float, all[6:])
        num_pts = len(all_lines[s][t]) 
        all_wt_values = []
        for i in range(num_pts):
            vals = []
            wvals = []
            wtot = 0
            eboltz = 0
            for s in all_stems:
                val = all_lines[s][t][i]
                vals.append(val)
                wval = math.e**(-1.*val/rt)
                wvals.append(wval)
                wtot+= wval #add this value
            for j in range(len(all_stems)):
                if wtot!=0:    #if it is, eboltz stays 0
                    eboltz += vals[j] * wvals[j]/wtot
            all_wt_values.append(eboltz)
        #default map_stem is 'weighted'
        fn = map_stem + '.'+ t +'.map'
        fptr = open(fn, 'w')
        for l in all[:6]:
            fptr.write(l)
        for v in all_wt_values:
            l = '%.3f\n'%v
            fptr.write(l)
        fptr.close()
        if verbose: print "wrote ", fn
    #------------------------------------------------------------------
    #   d,e maps
    #------------------------------------------------------------------
    for t in ['e','d']:
        for s in all_stems:  #loop over receptors
            all_lines[s] = {}
            fn = "%s.%s.map" %(s,t)
            if verbose: print "opening ", fn
            fptr = open(fn)
            all = fptr.readlines()
            fptr.close()
            #skip the first 6 lines of header and convert to float
            all_lines[s][t] = map(float, all[6:])
        #perhaps check this is the same as the first part
        #assert len(all_lines[s][t])==num_pts 
        num_pts = len(all_lines[s][t]) 
        all_wt_values = []
        ct = len(all_stems)
        for i in range(num_pts):
            #get list of all the values for this point
            pt_vals = []
            for s in all_stems:
                val = all_lines[s][t][i]
                pt_vals.append(val)
            #get list of absolute value of all the values for this point
            abs_values = map(abs, pt_vals)
            #get mininum of list of absolute values
            mval = min(abs_values)
            #get index of the mininum of list of absolute values
            ind = abs_values.index(mval)
            #get corresponding signed value
            all_wt_values.append(pt_vals[ind])
            #all_wt_values.append(eboltz)
        fn = map_stem + '.'+t+'.map'
        fptr = open(fn, 'w')
        for l in all[:6]:
            fptr.write(l)
        for v in all_wt_values:
            l = '%.3f\n'%v
            fptr.write(l)
        fptr.close()
        if verbose: print "wrote ", fn

    #------------------------------------------------------------------
    #   .fld and .xyz files: use last stem 's'
    #------------------------------------------------------------------
    #xyz file
    xyz_fn = s + '.maps.xyz'
    fptr = open(xyz_fn)
    lines = fptr.readlines()
    fptr.close()
    if verbose: print "read ", xyz_fn
    new_fn = map_stem + '.maps.xyz'
    optr = open(new_fn, 'w')
    for l in lines: 
        optr.write(l)
    optr.close()    
    if verbose: print "wrote ", new_fn
    #fld file
    fld_fn = s + '.maps.fld'
    fptr = open(fld_fn)
    lines = fptr.readlines()
    fptr.close()
    if verbose: print "read ", fld_fn
    new_fn = map_stem + '.maps.fld'
    optr = open(new_fn, 'w')
    for l in lines: 
        optr.write(l.replace(s, map_stem))
    optr.close()    
    if verbose: print "wrote ", new_fn



########################################################################
#
#    Vision Macro - Python source code - file generated by vision
#    Thursday 01 June 2006 11:29:09 
#    
#       The Scripps Research Institute (TSRI)
#       Molecular Graphics Lab
#       La Jolla, CA 92037, USA
#
# Copyright: Daniel Stoffler, Michel Sanner and TSRI
#   
# revision: Guillaume Vareille
#  
#########################################################################
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/VisionInterface/PrepareAD3Ligand.py,v 1.1 2006/06/05 22:45:41 rhuey Exp $
#
# $Id: PrepareAD3Ligand.py,v 1.1 2006/06/05 22:45:41 rhuey Exp $
#

from NetworkEditor.macros import MacroNode
class PrepareAD3Ligand(MacroNode):

    def __init__(self, constrkw={}, name='Prepare AD3 Ligand', **kw):
        kw['name'] = name
        apply( MacroNode.__init__, (self,), kw)

    def beforeAddingToNetwork(self, net):
        MacroNode.beforeAddingToNetwork(self, net)
        ## loading libraries ##
        from AutoDockTools.VisionInterface.AdtNodes import adtlib
        net.editor.addLibraryInstance(adtlib,"AutoDockTools.VisionInterface.AdtNodes", "adtlib")

        from MolKit.VisionInterface.MolKitNodes import molkitlib
        net.editor.addLibraryInstance(molkitlib,"MolKit.VisionInterface.MolKitNodes", "molkitlib")


    def afterAddingToNetwork(self):
        from NetworkEditor.macros import MacroNode
        MacroNode.afterAddingToNetwork(self)
        ## loading libraries ##
        from AutoDockTools.VisionInterface.AdtNodes import adtlib
        from MolKit.VisionInterface.MolKitNodes import molkitlib
        ## building macro network ##
        Prepare_AD3_Ligand_0 = self
        from traceback import print_exc

        ## loading libraries ##
        from AutoDockTools.VisionInterface.AdtNodes import adtlib
        self.macroNetwork.getEditor().addLibraryInstance(adtlib,"AutoDockTools.VisionInterface.AdtNodes", "adtlib")

        from MolKit.VisionInterface.MolKitNodes import molkitlib
        self.macroNetwork.getEditor().addLibraryInstance(molkitlib,"MolKit.VisionInterface.MolKitNodes", "molkitlib")

        try:
            ## saving node input Ports ##
            input_Ports_1 = self.macroNetwork.ipNode
            input_Ports_1.move(183, 27)
        except:
            print "WARNING: failed to restore MacroInputNode named input Ports in network self.macroNetwork"
            print_exc()
            input_Ports_1=None

        try:
            ## saving node output Ports ##
            output_Ports_2 = self.macroNetwork.opNode
            output_Ports_2.move(181, 338)
        except:
            print "WARNING: failed to restore MacroOutputNode named output Ports in network self.macroNetwork"
            print_exc()
            output_Ports_2=None

        try:
            ## saving node Calculate Gasteiger Charges ##
            from MolKit.VisionInterface.MolKitNodes import CalculateGasteigerCharges
            Calculate_Gasteiger_Charges_3 = CalculateGasteigerCharges(constrkw = {}, name='Calculate Gasteiger Charges', library=molkitlib)
            self.macroNetwork.addNode(Calculate_Gasteiger_Charges_3,200,131)
            apply(Calculate_Gasteiger_Charges_3.inputPortByName['mols'].configure, (), {'color': '#c64e70', 'cast': True, 'shape': 'oval'})
            apply(Calculate_Gasteiger_Charges_3.outputPortByName['mols'].configure, (), {'color': '#c64e70', 'shape': 'oval'})
            apply(Calculate_Gasteiger_Charges_3.outputPortByName['charge_total'].configure, (), {'color': 'green', 'shape': 'circle'})
        except:
            print "WARNING: failed to restore CalculateGasteigerCharges named Calculate Gasteiger Charges in network self.macroNetwork"
            print_exc()
            Calculate_Gasteiger_Charges_3=None

        try:
            ## saving node Add Hydrogens ##
            from MolKit.VisionInterface.MolKitNodes import AddHydrogens
            Add_Hydrogens_4 = AddHydrogens(constrkw = {}, name='Add Hydrogens', library=molkitlib)
            self.macroNetwork.addNode(Add_Hydrogens_4,200,80)
            apply(Add_Hydrogens_4.inputPortByName['molecules'].configure, (), {'color': '#c64e70', 'cast': True, 'shape': 'oval'})
            apply(Add_Hydrogens_4.outputPortByName['molecules'].configure, (), {'color': '#c64e70', 'shape': 'oval'})
            apply(Add_Hydrogens_4.outputPortByName['new_h_ct'].configure, (), {'color': 'yellow', 'shape': 'circle'})
        except:
            print "WARNING: failed to restore AddHydrogens named Add Hydrogens in network self.macroNetwork"
            print_exc()
            Add_Hydrogens_4=None

        try:
            ## saving node Merge NonPolar Hydrogens ##
            from AutoDockTools.VisionInterface.AdtNodes import MergeNonPolarHydrogens
            Merge_NonPolar_Hydrogens_5 = MergeNonPolarHydrogens(constrkw = {}, name='Merge NonPolar Hydrogens', library=adtlib)
            self.macroNetwork.addNode(Merge_NonPolar_Hydrogens_5,200,182)
            apply(Merge_NonPolar_Hydrogens_5.inputPortByName['mols'].configure, (), {'color': '#c64e70', 'cast': True, 'shape': 'oval'})
            apply(Merge_NonPolar_Hydrogens_5.inputPortByName['renumber'].configure, (), {'color': 'yellow', 'cast': True, 'shape': 'circle'})
            apply(Merge_NonPolar_Hydrogens_5.outputPortByName['mols'].configure, (), {'color': '#c64e70', 'shape': 'oval'})
            apply(Merge_NonPolar_Hydrogens_5.outputPortByName['num_nphs'].configure, (), {'color': 'yellow', 'shape': 'circle'})
        except:
            print "WARNING: failed to restore MergeNonPolarHydrogens named Merge NonPolar Hydrogens in network self.macroNetwork"
            print_exc()
            Merge_NonPolar_Hydrogens_5=None

        try:
            ## saving node Manage Rotatable Bonds ##
            from AutoDockTools.VisionInterface.AdtNodes import ManageRotatableBonds
            Manage_Rotatable_Bonds_6 = ManageRotatableBonds(constrkw = {}, name='Manage Rotatable Bonds', library=adtlib)
            self.macroNetwork.addNode(Manage_Rotatable_Bonds_6,200,235)
            apply(Manage_Rotatable_Bonds_6.inputPortByName['mols'].configure, (), {'color': '#c64e70', 'cast': True, 'shape': 'oval'})
            apply(Manage_Rotatable_Bonds_6.inputPortByName['root'].configure, (), {'color': 'white', 'cast': True, 'shape': 'oval'})
            apply(Manage_Rotatable_Bonds_6.inputPortByName['allowed_bonds'].configure, (), {'color': 'white', 'cast': True, 'shape': 'oval'})
            apply(Manage_Rotatable_Bonds_6.inputPortByName['check_for_fragments'].configure, (), {'color': 'yellow', 'cast': True, 'shape': 'circle'})
            apply(Manage_Rotatable_Bonds_6.inputPortByName['bonds_to_inactivate'].configure, (), {'color': 'white', 'cast': True, 'shape': 'oval'})
            apply(Manage_Rotatable_Bonds_6.inputPortByName['limit_torsions'].configure, (), {'color': 'white', 'cast': True, 'shape': 'oval'})
            apply(Manage_Rotatable_Bonds_6.outputPortByName['mol'].configure, (), {'color': '#c64e70', 'shape': 'oval'})
        except:
            print "WARNING: failed to restore ManageRotatableBonds named Manage Rotatable Bonds in network self.macroNetwork"
            print_exc()
            Manage_Rotatable_Bonds_6=None

        try:
            ## saving node AD4_typer ##
            from AutoDockTools.VisionInterface.AdtNodes import Assign_AD4Types
            AD4_typer_7 = Assign_AD4Types(constrkw = {}, name='AD4_typer', library=adtlib)
            self.macroNetwork.addNode(AD4_typer_7,198,288)
            apply(AD4_typer_7.inputPortByName['mols'].configure, (), {'color': '#c64e70', 'cast': True, 'shape': 'oval'})
            apply(AD4_typer_7.inputPortByName['renameAtoms'].configure, (), {'color': 'yellow', 'cast': True, 'shape': 'circle'})
            apply(AD4_typer_7.inputPortByName['set_aromatic_carbons'].configure, (), {'color': 'yellow', 'cast': True, 'shape': 'circle'})
            apply(AD4_typer_7.inputPortByName['reassign'].configure, (), {'color': 'yellow', 'cast': True, 'shape': 'circle'})
            apply(AD4_typer_7.outputPortByName['typed_mols'].configure, (), {'color': '#c64e70', 'shape': 'oval'})
        except:
            print "WARNING: failed to restore Assign_AD4Types named AD4_typer in network self.macroNetwork"
            print_exc()
            AD4_typer_7=None

        self.macroNetwork.freeze()

        ## saving connections for network Prepare AD3 Ligand ##
        input_Ports_1 = self.macroNetwork.ipNode
        if input_Ports_1 is not None and Add_Hydrogens_4 is not None:
            try:
                self.macroNetwork.connectNodes(
                    input_Ports_1, Add_Hydrogens_4, "new", "molecules", blocking=True)
            except:
                print "WARNING: failed to restore connection between input_Ports_1 and Add_Hydrogens_4 in network self.macroNetwork"
        if Add_Hydrogens_4 is not None and Calculate_Gasteiger_Charges_3 is not None:
            try:
                self.macroNetwork.connectNodes(
                    Add_Hydrogens_4, Calculate_Gasteiger_Charges_3, "molecules", "mols", blocking=True)
            except:
                print "WARNING: failed to restore connection between Add_Hydrogens_4 and Calculate_Gasteiger_Charges_3 in network self.macroNetwork"
        if Calculate_Gasteiger_Charges_3 is not None and Merge_NonPolar_Hydrogens_5 is not None:
            try:
                self.macroNetwork.connectNodes(
                    Calculate_Gasteiger_Charges_3, Merge_NonPolar_Hydrogens_5, "mols", "mols", blocking=True)
            except:
                print "WARNING: failed to restore connection between Calculate_Gasteiger_Charges_3 and Merge_NonPolar_Hydrogens_5 in network self.macroNetwork"
        if Merge_NonPolar_Hydrogens_5 is not None and Manage_Rotatable_Bonds_6 is not None:
            try:
                self.macroNetwork.connectNodes(
                    Merge_NonPolar_Hydrogens_5, Manage_Rotatable_Bonds_6, "mols", "mols", blocking=True)
            except:
                print "WARNING: failed to restore connection between Merge_NonPolar_Hydrogens_5 and Manage_Rotatable_Bonds_6 in network self.macroNetwork"
        if Manage_Rotatable_Bonds_6 is not None and AD4_typer_7 is not None:
            try:
                self.macroNetwork.connectNodes(
                    Manage_Rotatable_Bonds_6, AD4_typer_7, "mol", "mols", blocking=True)
            except:
                print "WARNING: failed to restore connection between Manage_Rotatable_Bonds_6 and AD4_typer_7 in network self.macroNetwork"
        output_Ports_2 = self.macroNetwork.opNode
        if AD4_typer_7 is not None and output_Ports_2 is not None:
            try:
                self.macroNetwork.connectNodes(
                    AD4_typer_7, output_Ports_2, "typed_mols", "new", blocking=True)
            except:
                print "WARNING: failed to restore connection between AD4_typer_7 and output_Ports_2 in network self.macroNetwork"
        self.macroNetwork.unfreeze()

        Prepare_AD3_Ligand_0.shrink()

        ## reset modifications ##
        Prepare_AD3_Ligand_0.resetTags()
        Prepare_AD3_Ligand_0.buildOriginalList()



#############################################################################
#
# Author: Ruth HUEY, Michel F. SANNER
#
# Copyright: M. Sanner TSRI 2008
#
#############################################################################

# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/autoflex41Commands.py,v 1.4 2013/05/09 17:01:27 rhuey Exp $ 
#
# $Id: autoflex41Commands.py,v 1.4 2013/05/09 17:01:27 rhuey Exp $
#
#
#
#
#
"""
This Module facilitates producing a formatted flexible residue file for AutoDock. The steps in this process are:

    * Set the macromolecule: 

        o Read a PDBQT Macromolecule 

        o Choose Macromol...

    * Select which residues are to be flexible in macromolecule using Pmv selection tools:

        o ICOM Select 

        o SelectFromString

        o Select Spherical Region

    * Set which torsions in the sidechains of those residues are to be flexible interactively

    * The results of the previous steps are written to two files: 

        o one containing the sidechains of the flexible residues with special keywords

        o a second containing the rigid portion of the macromolecule 
    
"""




from ViewerFramework.VFCommand import CommandGUI

from AutoDockTools.autoflexCommands import  AF_MacroReader,\
AF_MacroChooser, AF_SelectResidues, AF_ProcessResidues,\
AF_ProcessHingeResidues, AF_EditHinge, AF_SetHinge,\
AF_SetBondRotatableFlag, AF_StepBack, AF_FlexFileWriter,\
AF_RigidFileWriter, AF_LigandDirectoryWriter, AF_SetupCovalentFlexibleResidue,\
menuText


AF_MacroReaderGUI=CommandGUI()
AF_MacroReaderGUI.addMenuCommand('AutoTools41Bar', menuText['AutoFlexMB'], \
        menuText['Read Macro'], cascadeName = menuText['InputMB'])

AF_MacroChooserGUI=CommandGUI()
AF_MacroChooserGUI.addMenuCommand('AutoTools41Bar', menuText['AutoFlexMB'],
            menuText['Choose Macro'], cascadeName = menuText['InputMB'])

AF_SelectResiduesGUI = CommandGUI()
AF_SelectResiduesGUI.addMenuCommand('AutoTools41Bar', menuText['AutoFlexMB'],menuText['Set Residues'])

AF_ProcessResiduesGUI = CommandGUI()
AF_ProcessHingeResiduesGUI = CommandGUI()

AF_EditHingeGUI = CommandGUI()
AF_EditHingeGUI.addMenuCommand('AutoTools41Bar', menuText['AutoFlexMB'],\
        menuText['Edit Hinge'])

AF_SetHingeGUI = CommandGUI()
AF_SetHingeGUI.addMenuCommand('AutoTools41Bar', menuText['AutoFlexMB'],\
        menuText['Set Hinge'])

AF_SetupCovalentFlexibleResidueGUI = CommandGUI()
AF_SetupCovalentFlexibleResidueGUI.addMenuCommand('AutoTools41Bar', menuText['AutoFlexMB'],\
        menuText['Setup Covalent Residue'])

AF_StepBackGUI = CommandGUI()
AF_StepBackGUI.addMenuCommand('AutoTools41Bar', menuText['AutoFlexMB'], menuText['Step Back'])

AF_FlexFileWriterGUI = CommandGUI()
AF_FlexFileWriterGUI.addMenuCommand('AutoTools41Bar', menuText['AutoFlexMB'], \
            menuText['writeFlexible'], cascadeName = menuText['WriteMB'])

AF_RigidFileWriterGUI = CommandGUI()
AF_RigidFileWriterGUI.addMenuCommand('AutoTools41Bar',  menuText['AutoFlexMB'], \
            menuText['writeRigid'], cascadeName = menuText['WriteMB'])

AF_LigandDirectoryWriterGUI = CommandGUI()
AF_LigandDirectoryWriterGUI.addMenuCommand('AutoTools41Bar', menuText['AutoFlexMB'], \
            menuText['writeDir'], cascadeName = menuText['WriteMB'])


commandList = [
    {'name':'AD41flex_readMacro','cmd':AF_MacroReader(),'gui':AF_MacroReaderGUI},
    {'name':'AD41flex_chooseMacro','cmd':AF_MacroChooser(),'gui':AF_MacroChooserGUI},
    {'name':'AD41flex_setResidues','cmd':AF_SelectResidues(),'gui':AF_SelectResiduesGUI},
    #{'name':'AD41flex_processResidues','cmd':AF_ProcessResidues(),'gui':None},
    #{'name':'AD41flex_processHingeResidues','cmd':AF_ProcessHingeResidues(),'gui':None},
    {'name':'AD41flex_setupCovalentResidue', 'cmd':AF_SetupCovalentFlexibleResidue(), 'gui':AF_SetupCovalentFlexibleResidueGUI},
    #{'name':'AD41flex_setBondRotatableFlag','cmd':AF_SetBondRotatableFlag(),'gui':None},
    #{'name':'AD41flex_setHinge','cmd':AF_SetHinge(),'gui':AF_SetHingeGUI},
    #{'name':'AD41flex_editHinge','cmd':AF_EditHinge(),'gui':None},
    {'name':'AD41flex_stepBack','cmd':AF_StepBack(),'gui':AF_StepBackGUI},
    {'name':'AD41flex_writeFlexFile','cmd':AF_FlexFileWriter(),'gui':AF_FlexFileWriterGUI},
    {'name':'AD41flex_writeRigidFile','cmd':AF_RigidFileWriter(),'gui':AF_RigidFileWriterGUI},
    #{'name':'AD41flex_writeFlexDir','cmd':AF_LigandDirectoryWriter(),'gui':AF_LigandDirectoryWriterGUI}
]

def initModule(vf):
    for dict in commandList:
        vf.addCommand(dict['cmd'], dict['name'], dict['gui'])
    if not hasattr(vf, 'ADflex_processResidues'):
        vf.addCommand(AF_ProcessResidues(), 'ADflex_processResidues', None)
    if not hasattr(vf, 'ADflex_setBondRotatableFlag'):
        vf.addCommand(AF_SetBondRotatableFlag(), 'ADflex_setBondRotatableFlag', None)   
    vf.ADflex_setResidues = vf.AD41flex_setResidues
      

    
    if vf.hasGui and 'AutoTools41Bar' in vf.GUI.menuBars.keys():
        vf.GUI.menuBars['AutoTools41Bar'].menubuttons[menuText['AutoFlexMB']].config(bg='tan',underline='-1')    
        if not hasattr(vf.GUI, 'adtBar'):
            vf.GUI.adtBar = vf.GUI.menuBars['AutoTools41Bar']
            vf.GUI.adtFrame = vf.GUI.adtBar.menubuttons.values()[0].master



 






#!/usr/bin/env python
#
# 
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/Utilities24/prepare_gpf.py,v 1.6 2007/10/09 17:30:07 annao Exp $
#

import string
import os.path
from math import ceil
from MolKit import Read
from AutoDockTools.GridParameters import GridParameters, grid_parameter_list
from AutoDockTools.GridParameters import GridParameterFileMaker


def usage():
    print "Usage: prepare_gpf.py -l pdbq_file -r pdbqs_file "
    print "     -l ligand_filename"
    print "     -r receptor_filename"
    print
    print "Optional parameters:"
    print "    [-i reference_gpf_filename]"
    print "    [-o output_gpf_filename]"
    print "    [-p parameter=newvalue]"
    print "    [-v]"
    print
    print "Prepare a grid parameter file (GPF) for AutoDock."
    print
    print "   The GPF will by default be <receptor>.gpf. This"
    print "may be overridden using the -o flag."

    
if __name__ == '__main__':
    import getopt
    import sys

    try:
        opt_list, args = getopt.getopt(sys.argv[1:], 'vl:r:i:o:p:')
    except getopt.GetoptError, msg:
        print 'prepare_gpf.py: %s' % msg
        usage()
        sys.exit(2)

    receptor_filename = ligand_filename = None
    list_filename = gpf_filename = gpf_filename = None
    output_gpf_filename = None
    parameters = []
    verbose = None
    for o, a in opt_list:
        if o in ('-v', '--v'):
            verbose = 1
        if o in ('-l', '--l'):
            ligand_filename = a
            if verbose: print 'ligand_filename=', ligand_filename
        if o in ('-r', '--r'):
            receptor_filename = a
            if verbose: print 'receptor_filename=', receptor_filename
        if o in ('-i', '--i'):
            gpf_filename = a
            if verbose: print 'reference_gpf_filename=', gpf_filename
        if o in ('-o', '--o'):
            output_gpf_filename = a
            if verbose: print 'output_gpf_filename=', output_gpf_filename
        if o in ('-p', '--p'):
            parameters.append(a)
            if verbose: print 'parameters=', parameters
        if o in ('-h', '--'):
            usage()
            sys.exit()


    if (not receptor_filename) or (not ligand_filename):
        print "prepare_gpf.py: ligand and receptor filenames"
        print "                    must be specified."
        usage()
        sys.exit()

    gpfm = GridParameterFileMaker(verbose=verbose)
    gpfm.set_ligand(ligand_filename)
    gpfm.set_receptor(receptor_filename)
    if gpf_filename is not None:
        gpfm.read_reference(gpf_filename)
    for p in parameters:
        key,newvalue = string.split(p, '=')
        kw = {key:newvalue}
        apply(gpfm.set_grid_parameters, (), kw)
    #gpfm.set_grid_parameters(spacing=1.0)
    gpfm.write_gpf(output_gpf_filename)


#prepare_gpf.py -l indinavir.pdbq -r 1hsg.pdbqs -p spacing=0.4 -p npts=[60,60,60] -i ref.gpf -o testing.dpf 




#############################################################################
#
# Author: J. Ren
#
#############################################################################

"""
TBD
"""

import os

class receptor_prepared:

    def __init__(self, receptor_path=None):

        if receptor_path.startswith('http://'):
            self.type = 'url'
            self.path = receptor_path
        else:
            self.type = 'local'
            self.path = os.path.abspath(receptor_path)

        self.basename = os.path.basename(receptor_path)
        self.id = self.basename.rstrip('.pdbqt')



"""
Authors: Stefano Forli, M Sanner

April 2010

copyright TSRI

CalculateProperties can be used to process a directory and create a property
table for the ligands in that directory.

example:
    calcProp = CalculateProperties(verbose=2)
    propT = calcProp.processDirectory(mydir, '*.pdbqt')

    or for a folder hirearchy
    propT = calcProp.processDirectoryTree(root, extension='.pdbqt')

propT is a PropertyTable object that stores the directory where the files are
and has a table attribute containing the actual properties.

propT.saveAsTextFile(filename) saves the property table

A PropertyTable object can be recreated from a file:
   pt = PropertyTable(filename='lib1.prop')

or from a URL
   pt = PropertyTable(url='http://www.scripps.edu/~sanner/collab/lib1.prop')


The FilterLigands can be used to filter ligands based on properties.
It creates a LigandList object which contains a list of full names (i.e.
path and file name) for each seelcted ligand. using full names enables
combining molecules from multiple databases, This object provides several
pre-set ranges of values for the properties corresponding to 4 commonly used
filters.

example:
    lfilter = FilterLigands()
    lfilter.setFilter('default')
    accepted, rejected = lfilter.filterTable(pt)
    print 'default filter in %f second accepted %d rejected %d'%(
        lfilter.filteringTime, len(accepted), len(rejected))
    print lfilter.getAtomTypes(calcProp.propertyTable, accepted)

"""

class PropertyTable:
    """
    """

    def __init__(self, folder=None, table=None, url=None, filename=None):
        self.folder = folder     # folder containing the ligand files
        self.table = table       #
        self.url = url        # folder containing the ligand files
        self.filename = filename   # folder containing the ligand files
        if url is not None:
            self.fillFromURL(url)
        elif filename is not None:
            self.fillFromFile(filename)
        else:
            assert folder is not None and isinstance(table, dict)


    def saveAsTextFile(self, filename):
        """
        None <- CalculateProperties.saveAsTextFile(filename)

        saves the propertyTable in a file as a dictionary called propertyTable

        """ 
        TORSDOF = 0
        HbD = 1
        HbA = 2
        MW  = 3
        Nat = 4
        NotStdAT = 5
        Atypes = 6
        nameLength = max( map(len, self.table.keys()))
        f = open(filename, 'w')
        pad = max(0, nameLength-7)
        f.write('folder %s\n'%self.folder)
        f.write('# file '+' '*pad+' #TORS HbD HbA  MW   Nat nstd Types\n')
        fmt = "%"+str(nameLength)+"s %3d  %3d %3d %5.2f  %3d  %d"
        for key, v in self.table.items():
            f.write(fmt%(key, v[TORSDOF], v[HbD], v[HbA], v[MW],
                    v[Nat], v[NotStdAT]))
            for t in v[Atypes]:
                f.write(' %2s'%t)
            f.write("\n")
        f.close()


    def fillFromFile(self, filename):
        f = open(filename)
        lines = f.read().split('\n')
        f.close()
        self._parseLines(lines)


    def fillFromURL(self, url):
        import urllib
        
        fp = urllib.urlopen(url)
        lines = fp.read().split('\n')
        fp.close()
        self._parseLines(lines)


    def _parseLines(self, lines):
        propTable = {}
        folder = None
        for line in lines:
            if len(line)==0 or line[0]=='#':
                continue
            elif line[:6]=='folder':
                self.folder = line[6:].strip()
            else:
                w = line.split()
                propTable[w[0]] = [ int(w[1]), int(w[2]), int(w[3]),
                                    float(w[4]), int(w[5]), int(w[6]), w[7:] ]
        self.table = propTable

##     def saveAsPythonCode(self, filename):
##         """
##         None <- CalculateProperties.saveAsPythonCode(filename)

##         filename isa to have a .py extension
##         saves the propertyTable in a file as a dictionary called propertyTable
##         """
##         from os import path
##         assert path.splitext(filename)[1]=='.py'
##         f = open(filename, 'w')
##         f.write('propertyTable = ')
##         f.write(str(self.table))
##         f.close()


class CalculateProperties:
    """
    This function parses the lines from a pdbqt file provided in records
    and creates a property record for the this molecule.
    the property record contains the following information:
        TORSDOF: int - number of rotatable bonds
        atype: list of strings - list of tom types in the molecule
        HbD: int - number of hydrogen bond donors
        HbA: int - number of hydrogen bond acceptors
        MW: float - molecular weight
        Nat: int - number of atoms
        NotStdAT: bool - contains non standard atoms

    the property records are stored in the propertyTable using the filename as
    as key
    
    Usage:
        cp = CalculateProperties(verbose=0)
        cp.processDirectory(dirname, '*.pdbqt')

    """
    
    molecularWeight = {   # MW
        'H'      :   1   ,
        'HD'     :   1   ,
        'HS'     :   1   ,
        'C'      :   12  ,
        'A'      :   12  ,
        'N'      :   14  ,
        'NA'     :   14  ,
        'NS'     :   14  ,
        'OA'     :   16  ,
        'OS'     :   16  ,
        'F'      :   19  ,
        'Mg'     :   24  ,
        'MG'     :   24  ,
        'P'      :   31  ,
        'SA'     :   32  ,
        'S'      :   32  ,
        'Cl'     :   35.4,
        'CL'     :   35.4,
        'Ca'     :   40  ,
        'CA'     :   40  ,
        'Mn'     :   55  ,
        'MN'     :   55  ,
        'Fe'     :   56  ,
        'FE'     :   56  ,
        'Zn'     :   65.4,
        'ZN'     :   65.4,
        'Br'     :   80  ,
        'BR'     :   80  ,
        'I'      :  126  ,
        'e'      :    0  ,
        'd'      :    0   
        }


    def __init__(self, verbose=0):
        """
        Constructor

        if verbose > 1: processing time will be printed
        if verbose > 10: properties for each molecule willbe printed
        """
        self.verbose = verbose
        self.pocessingTime = 0.0
        self.directory = None
        
        
    def getRecords(self, filename):
        """
        records <- CalculateProperties.getRecords(filename)

        read the file and returns a list of lines in the file
        """
        file = open(filename, 'r')
        lines = file.readlines()
        file.close()
        return lines


    def processDirectoryTree(self, root, extension='.pdbqt'):
        """
        PropertyTable <- CalculateProperties.processDirectoryTree(root, extension='.pdbqt')
        find all files matching extension in the folder tree starting at root
        """
        import os, time
        
        splitext = os.path.splitext
        join = os.path.join
        t1 = time.time()
        propT = {}
        off = len(root)+1
        for rootl, dirs, files in os.walk(root):
            for f in files:
                if splitext(f)[1]==extension:
                    filename = join(rootl, f)
                    lines = self.getRecords(filename)
                    key = filename[off:] # exclude root
                    self.calcProp(lines, key, propT)
                    
        propTab = PropertyTable(folder=root, table=propT)

        self.processingTime = time.time()-t1
        if self.verbose>1:
            print 'Processed %d molecules in %.2f seconds'%(
                len(propT), self.processingTime)
        return propTab


    def processDirectory(self, dirname, selector='*'):
        """
        PropertyTable <- CalculateProperties.processDirectory(dirname, selector='*')
        """
        from glob import glob
        import os, time
        
        t1 = time.time()
        propT = {}
        filenames = glob(os.path.join(dirname, selector))
        for filename in filenames:
            lines = self.getRecords(filename)
            # the the filename without extension as the key
            key = os.path.splitext(os.path.basename(filename))[0]
            self.calcProp(lines, key, propT)

        propTab = PropertyTable(folder=dirname, table=propT)

        self.processingTime = time.time()-t1
        if self.verbose>1:
            print 'Processed %d molecules in %.2f seconds'%(
                len(propT), self.processingTime)
        return propTab
    

    def calcProp(self, records, key, propT):
        """
        """
        current_atypes = {}
        BAD_ATOM_TYPE = False
        MW  = 0
        HbD = 0
        HbA = 0
        Nat = 0
        status = True
        TORSDOF = 0

        hbd_h = []
        hbd_candidate = []

        # Calculate all the properties
        for line in records:
            if 'TORSDOF' in line: # not a defined location ??
                TORSDOF = int(line[8:])

            if line[0:6] == 'HETATM' or line[0:4] == 'ATOM':
                # Nat += 1 # Consider to remove Hydrogens? (search on PubMed)
                atype = line[76:-1].strip()
                if atype not in current_atypes:
                    current_atypes[atype] = True

                # Hb acceptor
                if atype == "OA" or atype == "NA" or atype == "SA":
                    HbA += 1

                # Hb donor preparation
                if atype == "HD":
                    #capture the hydrogens that could be bond to the Hb donor...
                    hbd_h.append( (float(line[30:38]), float(line[38:46]),
                                   float(line[46:54]) ) )
                else:
                    # count heavy atoms
                    Nat += 1

                if atype == "N" or atype == "O" or atype == "OA" or atype == "NA":
                    hbd_candidate.append( (float(line[30:38]),
                                   float(line[38:46]), float(line[46:54])) )
                try:
                    # add the atomic weight to the total MW
                    MW += self.molecularWeight[atype]
                except:
                    #MW += 10000 # check this if it's reasonable
                    MW += 0 # check this if it's reasonable
                    BAD_ATOM_TYPE = True
                    status = False # non-standard atom types are rejected by default

            # identify HBD by checking if N/O's are bound to H's
            dcut2 = 1.1 *1.1
            for atom in hbd_candidate:
                x1,y1,z1 = atom
                for hydrogen in hbd_h:
                    x2,y2,z2 = hydrogen
                    d2 = (x2-x1)*(x2-x1) + (y2-y1)*(y2-y1) + (z2-z1)*(z2-z1) 
                    if d2 <= dcut2:
                        HbD += 1
                        break

        propT[key] = [ TORSDOF, HbD, HbA, MW, Nat, BAD_ATOM_TYPE, 
                       current_atypes.keys() ]
#            {
#            "Atypes": current_atypes.keys(),
#            "TORSDOF": TORSDOF,
#            "HbD": HbD,
#            "HbA": HbA,
#            "MW": MW,
#            "Nat": Nat,
#            "NotStdAT":BAD_ATOM_TYPE
#            }

        if self.verbose > 10:
            print "tors %d hbd:%d hba: %d mw: %.2f nat:%d nstd: %d, types: %s\n"%(
                TORSDOF, HbD, HbA, MW, Nat, BAD_ATOM_TYPE,
                current_atypes.keys()) 
           

       


class FilterLigands:
    """
    Class for filterting a propertyTable

    usage:
        ligFilter = FilterLigands()
        ligFilter.setFilter('default')
        accepted, rejected = ligFilter.filterTable(propertyTable)

    setFilter() accepts the following filters:
        'default', 'Lipinski-like', 'Drug-like', 'Drug-like frag'
        It also can be passed 'None' or None to prevent filtering

    setFilterRanges(filterType, **kw) can be called to set filtering values
    to special ranges.The .filter attribute is set to 'Custom' in this case.
    """
    filterNames = ['default', 'Lipinski-like', 'Drug-like', 'Drug-like frag']

    def __init__(self):
        self.filter = None # stores the filter type
        self.setFilter('default')
        self.rejectNStdAt = True
        self.filteringTime = 0


    def setFilterRanges(self, filterType, **kw):

        if filterType in self.filterNames+['None', None]:
            self.filter=filterType
        else:
            self.filter = 'Custom'

        if len(kw)==0: # no options means reset values to default
            self.HbDMin = 0# HydBond DONORS
            self.HbDMax = 99
            self.HbAMin = 0 # HydBond ACCEPTOR
            self.HbAMax = 99
            self.MWMin = 0 # Molecular weight
            self.MWMax = 99999
            self.NatMin = 0 # Number of heavy atoms
            self.NatMax = 999
            self.TORSDOFMin = 0
            self.TORSDOFMax = 32
        else:
            for key, value in kw.items():
                if key=='HbDMin':
                    self.HbDMin = value
                elif key=='HbDMax':
                    self.HbDMax = value
                elif key=='HbAMin':
                    self.HbAMin = value
                elif key=='HbAMax':
                    self.HbAMax = value
                elif key=='MWMin':
                    self.MWMin = value
                elif key=='MWMax':
                    self.MWMax = value
                elif key=='NatMin':
                    self.NatMin = value
                elif key=='NatMax':
                    self.NatMax = value
                elif key=='TORSDOFMin':
                    self.TORSDOFMin = value
                elif key=='TORSDOFMax':
                    self.TORSDOFMax = value
                else:
                    raise ValueError("bad key %s for setting value range, expected: HHbDMin, HbDMax, HbAMin, HbAMax, MWMin, MWMax, NatMin, NatMax, TORSDOFMin, TORSDOFMax")


    def setFilter(self, mode='default'):
        """
        Configure the filtering ranges for various types of filters
        mode can be 'default', 'Lipinski-like', 'Drug-like', or 'Drug-like frag'
        """
        if mode=='default':
            self.setFilterRanges( mode,
                HbDMin=0, HbDMax=99, HbAMin=0, HbAMax=99, MWMin=0, MWMax=9999,
                NatMin=0, NatMax=999, TORSDOFMin=0, TORSDOFMax=32)

        elif mode=='Lipinski-like':
            # http://en.wikipedia.org/wiki/Lipinski%27s_Rule_of_Five
            self.setFilterRanges( mode,
                HbDMin=0, HbDMax=5,HbAMin=0, HbAMax=10, MWMin=0, MWMax=500,
                NatMin=0, NatMax=999, TORSDOFMin=0, TORSDOFMax=32)

        elif mode=='Drug-like':
            # http://en.wikipedia.org/wiki/Lipinski%27s_Rule_of_Five#cite_note-2
            self.setFilterRanges( mode,
                HbDMin=0, HbDMax=5,HbAMin=0, HbAMax=10, MWMin=160, MWMax=480,
                NatMin=20, NatMax=70, TORSDOFMin=0, TORSDOFMax=32)

        elif mode=='Drug-like frag':
            # Values from Fattori's paper
            self.setFilterRanges( mode,
                HbDMin=0, HbDMax=3, HbAMin=0, HbAMax=6, MWMin=160, MWMax=300,
                NatMin=6, NatMax=45, TORSDOFMin=0, TORSDOFMax=32)

        elif mode=='None' or mode==None:
            # Values from Fattori's paper
            self.setFilterRanges( mode,
                HbDMin=None, HbDMax=None, HbAMin=None, HbAMax=None,
                MWMin=None, MWMax=None, NatMin=None, NatMax=None,
                TORSDOFMin=None, TORSDOFMax=None)
        else:
            raise ValueError('BAD FILTER: got %s, expected one of %s'%(
                mode, str(self.filterNames)))


    def filterTable(self, propT, subset=None):
        """
        filter ligands

        LigandList <- FilterLigands.filterTable(propertyTable, subset=None)
        
        is subset is not specified all keys in propertyTables will be
        considered else, only the listed set of keys will be considered for \
        filtering
        """
        from os.path import join
	accepted = []
	rejected = []

        HbAMin = self.HbAMin
        HbAMax = self.HbAMax
        HbDMin = self.HbDMin
        HbDMax = self.HbDMax
        MWMin = self.MWMin
        MWMax = self.MWMax
        NatMin = self.NatMin
        NatMax = self.NatMax
        TORSDOFMin = self.TORSDOFMin
        TORSDOFMax = self.TORSDOFMax

        import os
#        dirname = os.path.basename(os.path.abspath(propT.folder))
        dirname = ""
        table = propT.table
        
        # get all keys in case none were given
        if subset is None:
            subset = table.keys()

        from time import time
        t1 = time()

        if self.filter==None or self.filter=='None':
            accepted = subset
        else:
            # column numbers
            TORS = 0
            HbD  = 1
            HbA  = 2
            MW   = 3
            Nat  = 4
            nstd = 5

            for key in subset:
                v = table[key]
                if self.rejectNStdAt and v[nstd]:
                    rejected.append(key)
                else:
                    lhba =  v[HbA]
                    lhbd =  v[HbD]
                    ltors = v[TORS]
                    lmw =   v[MW]
                    lnat =  v[Nat]
                    if lhba<HbAMin or lhba>HbAMax or \
                       lhbd<HbDMin or lhbd>HbDMax or \
                       ltors<TORSDOFMin or ltors>TORSDOFMax or \
                       lmw<MWMin or lmw>MWMax or \
                       lnat<NatMin or lnat>NatMax:
                        rejected.append(key)
                    else:
                        accepted.append(key)

        self.filteringTime = time()-t1
        at = self.getAtomTypes(propT, accepted)

        # add dirname
        fullkey = [ join(dirname, x) for x in accepted ]
        acceptedList = LigandList( fullkey, at)
        return acceptedList, rejected


    def getAtomTypes(self,  propertyTable, subset=None):

        table = propertyTable.table
        # get all keys in case none were given
        if subset is None:
            subset = table.keys()

        atypes = {}
        
        for key in subset:
            for t in table[key][6]:
               atypes[t] = 1
        return atypes.keys()
    

class LigandList:
    """
    This class represents a list of ligands to be used in a virtual screening
    experiment
    """

    def __init__(self, filename, atomTypes):

        self.filenames = filename # list of filenames containing ligands
        self.atomTypes = atomTypes


    def addAtomTypes(self, atypes):
        for t in atypes:
            if t not in self.atomTypes:
                self.atomTypes.append(t)

if __name__=='__main__':
    import sys, os, time
    print sys.argv[1]
    calcProp = CalculateProperties(verbose=2)
    propT = calcProp.processDirectory(sys.argv[1], "*.pdbqt")

    #propT.saveAsPythonCode('lib1_prop.py')
    propT.saveAsTextFile('lib1.prop')

    # create the PropertyTable from a file
    pt = PropertyTable(filename='lib1.prop')
    
    # create the PropertyTable from a URL
    pt = PropertyTable(url="http://www.scripps.edu/~sanner/collab/lib1.prop")

    lfilter = FilterLigands()
    lfilter.setFilter('None')
    accepted, rejected = lfilter.filterTable(pt)
    print 'No filter in %f second accepted %d rejected %d'%(
        lfilter.filteringTime, len(accepted.filenames), len(rejected))
    print accepted.atomTypes

    lfilter.setFilter('default')
    accepted, rejected = lfilter.filterTable(pt)
    print 'default filter in %f second accepted %d rejected %d'%(
        lfilter.filteringTime, len(accepted.filenames), len(rejected))
    print accepted.atomTypes

    lfilter.setFilter('Lipinski-like')
    accepted, rejected = lfilter.filterTable(pt)
    print 'Lipinski filter in %f second accepted %d rejected %d'%(
        lfilter.filteringTime, len(accepted.filenames), len(rejected))
    print accepted.atomTypes
    
    lfilter.setFilter('Drug-like')
    accepted, rejected = lfilter.filterTable(pt)
    print 'Drug filter in %f second accepted %d rejected %d'%(
        lfilter.filteringTime, len(accepted.filenames), len(rejected))
    print accepted.atomTypes
    
    lfilter.setFilter('Drug-like frag')
    accepted, rejected = lfilter.filterTable(pt)
    print 'Drug frag filter in %f second accepted %d rejected %d'%(
        lfilter.filteringTime, len(accepted.filenames), len(rejected))
    print accepted.atomTypes

##     raise
##     # chaining filters
##     lfilter.setFilter('Lipinski-like')
##     accepted1, rejected1 = lfilter.filterTable(calcProp.propertyTable)

##     lfilter.setFilter('Drug-like frag')
##     accepted, rejected = lfilter.filterTable(calcProp.propertyTable,
##                                              subset=accepted1)
##     print 'Lipinski and Drug frag filter in %f second accepted %d rejected %d'%(
##         lfilter.filteringTime, len(accepted), len(rejected))
##     print lfilter.getAtomTypes(calcProp.propertyTable, accepted)

    



#!/usr/bin/env python

import os
from string import find

from MolKit import Read
from MolKit.pdbWriter import PdbqtWriter
from MolKit.chargeCalculator import GasteigerChargeCalculator
from AutoDockTools.atomTypeTools import AutoDock4_AtomTyper




if __name__ == '__main__':
    import sys
    import getopt


    def usage():
        "Print helpful, accurate usage statement to stdout."
        print "Usage: pdbqs_to_pdbqt.py -s filename"
        print
        print "    Description of command..."
        print "        [-s]    receptor_filename_stem"
        print "    Optional parameters:"
        print "        [-C]    preserve input charges, (default is addition of gasteiger charges)"   
        print "        [-p]    preserve input charges on atom type, eg -p Zn"
        print "        [-o]    alternative pdbqt_filename"
        print "        [-v]    verbose output"


    # process command arguments
    try:
        opt_list, args = getopt.getopt(sys.argv[1:], 's:Cp:o:v')

    except getopt.GetoptError, msg:
        print 'pdbqs_to_pdbqt.py: %s' %msg
        usage()
        sys.exit(2)

    charges_to_add = 'gasteiger'
    # initialize required parameters
    #-s: pdbqs_filename_stem
    pdbqs_filename =  None

    # optional parameters
    verbose = None
    pdbqt_filename =  None
    #-C do not add charges
    #-p preserve charges on specific atom types
    preserve_charge_types=''
    #'s:Cp:o:v'
    for o, a in opt_list:
        if o in ('-s', '--s'):
            pdbqs_filename = a + ".pdbqs"
            pdbqt_filename = a + ".pdbqt"
            if verbose: print 'set receptor_filename to ', pdbqs_filename
        if o in ('-C', '--C'):
            charges_to_add = None
            if verbose: print 'do not add charges'
        if o in ('-p', '--p'):
            preserve_charge_types+=a
            preserve_charge_types+=','
            if verbose: print 'preserve initial charges on ', preserve_charge_types
        if o in ('-o', '--o'):
            pdbqt_filename = a 
            if verbose: 
                print 'set pdbqt_filename to ', a
        if o in ('-v', '--v'):
            verbose = True
            if verbose: print 'set verbose to ', True
        if o in ('-h', '--'):
            usage()
            sys.exit()


    if not pdbqs_filename:
        print 'pdbqs_to_pdbqt: pdbqs_filename_stem must be specified.'
        usage()
        sys.exit()

    #what about nucleic acids???
    mols = Read(pdbqs_filename)
    if verbose: print 'read ', pdbqs_filename
    mol = mols[0]
    #fix number for problem files with alternative positions
    mol.allAtoms.number = range(1, len(mol.allAtoms)+1)
    #possible clean-up???
    #need to type atoms + assign babel_types
    AD4_typer = AutoDock4_AtomTyper(set_aromatic_carbons=False)
    AD4_typer.setAutoDockElements(mol)
    if verbose:
        print "set autodock4 autodock_element for ", mol.name
    if charges_to_add is not None:
        preserved = {}
        preserved_types = preserve_charge_types.split(',') 
        for t in preserved_types:
            if not len(t): continue
            ats = mol.allAtoms.get(lambda x: x.autodock_element==t)
            for a in ats:
                if a.chargeSet is not None:
                    preserved[a] = [a.chargeSet, a.charge]

    if charges_to_add=='gasteiger':
        GasteigerChargeCalculator().addCharges(mol.allAtoms)

    if charges_to_add is not None:
        #restore any previous charges
        for atom, chargeList in preserved.items():
            atom._charges[chargeList[0]] = chargeList[1]
            atom.chargeSet = chargeList[0]

    #pdbqt_filename = mol.name + '.pdbqt'
    writer = PdbqtWriter()
    fptr = open(pdbqt_filename, 'w')
    ctr = 0
    for line in mol.parser.allLines:
        if find(line, 'ATOM')<0 and find(line, "HETA")<0:
            fptr.write(line)
        else:
            this_atom = mol.allAtoms[ctr]
            writer.write_atom(fptr, this_atom)
            ctr = ctr + 1
    fptr.close()
    if verbose:
        print "wrote ", ctr, " atoms to", pdbqt_filename
    

# To execute this command type:
# pdbqs_to_pdbqt.py -s pdbqs_filename_stem  -v







#!/usr/bin/env python
#
# 
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/Utilities24/summarize_epdb_results4.py,v 1.2 2010/02/03 00:08:25 rhuey Exp $
#
# $Id: summarize_epdb_results4.py,v 1.2 2010/02/03 00:08:25 rhuey Exp $
#
import os, glob
from numpy import oldnumeric as Numeric
from AutoDockTools.EpdbParser import EpdbParser



if __name__ == '__main__':
    import sys
    import getopt


    def usage():
        "Print helpful, accurate usage statement to stdout."
        print "Usage: summarize_epdb_results4.py -f epdb_dlgfilename"
        print
        print "    Description of command..."
        print "         -f     epdb_dlgfilename"
        print "    Optional parameters:"
        print "        [-o]    output filename"
        print "                      (default is 'summary_of_epdb_results')"
        print "        [-a]    append to  output filename"
        print "                      (default is to open output filename 'w')"
        print "        [-v]    verbose output"


    # process command arguments
    try:
        opt_list, args = getopt.getopt(sys.argv[1:], 'f:o:avh')
    except getopt.GetoptError, msg:
        print 'summarize_epdb_results4.py: %s' %msg
        usage()
        sys.exit(2)

    # initialize required parameters
    #-f: epdb_dlgfilename
    epdb_dlgfilename =  None
    # optional parameters
    #-o outputfilename
    outputfilename = "summary_of_epdb_results"
    #-a append to outputfile
    append_to_outputfile = False
    #-v verbose
    verbose = False

    #'f:o:avh'
    for o, a in opt_list:
        #print "o=", o, " a=", a
        if o in ('-f', '--f'):
            epdb_dlgfilename = a
            if verbose: print 'set epdb_dlgfilename to ', a
        if o in ('-o', '--o'):
            outputfilename = a
            if verbose: print 'set outputfilename to ', a
        if o in ('-a', '--a'):
            append_to_outputfile = True
            if verbose: print 'set append_to_outputfile to ', True
        if o in ('-v', '--v'):
            verbose = True
            if verbose: print 'set verbose to ', True
        if o in ('-h', '--'):
            usage()
            sys.exit()

    if not  epdb_dlgfilename:
        print 'summarize_epdb_results4: epdb_dlgfilename must be specified.'
        usage()
        sys.exit()

    ep = EpdbParser(epdb_dlgfilename)
    mode = 'w'
    first = True
    if append_to_outputfile:
        mode = 'a'
        first = not os.path.exists(outputfilename)
    fptr = open(outputfilename, mode)

    if verbose: print "first is ", first
    if first:
        tstr = "      dlgfn      estFE  IntermolE   InternalE     torsFE   sumTotal   sumEstat     sumVDW  #at #tors\n "
        fptr.write(tstr)
    ostr = "%10s,%10.2f,%10.2f,%11.2f,%10.2f,%10.2f,%10.2f,%10.2f,%4d,%5d\n"%(os.path.basename(epdb_dlgfilename), \
                                                                          ep.estFreeEnergy, ep.finalIntermolEnergy, \
                                                                          ep.finalTotalInternalEnergy, ep.torsionalFreeEnergy, \
                                                                          Numeric.add.reduce(ep.total_energies), \
                                                                          Numeric.add.reduce(ep.estat_energies), \
                                                                          Numeric.add.reduce(ep.vdw_energies), ep.atmCtr, ep.ntors)
    fptr.write(ostr)
    fptr.close()
    
    

# To execute this command type:
# summarize_epdb_results4.py -d directory -t rmsd tolerance 
# -f epdb_dlgfilename
# -o outputfilename
# -a append to outputfilename
# -v verbose output



#$Header: /opt/cvs/python/packages/share1.5/AutoDockTools/WebServices.py,v 1.21 2009/08/12 21:40:05 lclement Exp $
#$Id: WebServices.py,v 1.21 2009/08/12 21:40:05 lclement Exp $
# Author: Sargis Dallakyan (sargis@scripps.edu)

import Tkinter, Pmw, os, httplib, webbrowser, urllib, re
from ViewerFramework.VFCommand import CommandGUI, Command
from Pmv.mvCommand import MVCommand
from autostartCommands import menuText
from mglutil.gui.InputForm.Tk.gui import InputFormDescr
from mglutil.util.packageFilePath import getResourceFolderWithVersion
from tkFileDialog import *
from tkMessageBox import *
from mglutil.gui.BasicWidgets.Tk.progressBar import ProgressBar
from mglutil.web.services.AppService_client import AppServiceLocator, launchJobRequest, \
getOutputsRequest, queryStatusRequest
from mglutil.web.services.AppService_types import ns0
import os

class WebServices(MVCommand):
    def __init__(self):
        MVCommand.__init__(self)
        rc =  getResourceFolderWithVersion() + os.sep + 'ws' + os.sep
        if not os.path.exists(rc):
            os.mkdir(rc)
        self.proxy_gama = rc + 'proxy_gama'
        self.rc_ad = rc + "rc_ad"
        self.login = False
        if hasattr(self, 'vf.GUI.ROOT'):
            self.dpf = Tkinter.StringVar(self.vf.GUI.ROOT)
            self.gpf = Tkinter.StringVar(self.vf.GUI.ROOT)
            self.prev_dir = Tkinter.StringVar(self.vf.GUI.ROOT)
            self.ad_radio = Tkinter.IntVar(self.vf.GUI.ROOT)
        else:
            self.dpf = Tkinter.StringVar()
            self.gpf = Tkinter.StringVar()
            self.prev_dir = Tkinter.StringVar()
            self.ad_radio = Tkinter.IntVar()
        self.current_job = None
            
    def guiCallback(self, event=None):
        mainform = self.showForm('default', modal=0, blocking=1.,
                                  initFunc=self.initForm)
         
    def buildFormDescr(self, formName):
        ifd = InputFormDescr(title = "AutoGrid/AutoDock Web Services")
        #Web Services Login
        ifd.append({'name':"LoginGroup", 'widgetType':Pmw.Group,
                     'container':{'LoginGroup':'w.interior()'},
                     'wcfg':{'tag_text':'Web Services Location'},
                     'gridcfg':{'sticky':'nswe'}
                    })
        ifd.append({'widgetType':Pmw.ComboBox, 'name':'WS_address',
                    'parent':'LoginGroup',
                    'wcfg':{'scrolledlist_items':
                            ('http://ws.nbcr.net/opal2/services',),
                            'listheight':50, 'dropdown':1, 'history':1, 
                            'autoclear':1},
                     'gridcfg':{'sticky':'ew', 'row':0, 'column':0,
                                'columnspan':3}
                     })

#        ifd.append({'widgetType':Tkinter.Label, 'name':'New_User',
#                    'parent':'LoginGroup', 'wcfg':{'text':'   New Users?',
#                                           'fg':'Blue','cursor':'hand1'},
#                    'gridcfg':{'sticky':'w', 'row':1, 'column':0}
#                    })
#        ifd.append({'widgetType':Tkinter.Label, 'name':'UserName_Label',
#                    'parent':'LoginGroup', 'wcfg':{'text':'User Name'},
#                    'gridcfg':{'sticky':'e', 'row':1, 'column':1}
#                    })
#        ifd.append({'widgetType':Tkinter.Entry, 'name':'UserName_Entry',
#                    'parent':'LoginGroup','wcfg':{},
#                    'gridcfg':{'sticky':'ew', 'row':1, 'column':2}
#                    })
#        ifd.append({'widgetType':Tkinter.Label, 'name':'Password_Label',
#                    'parent':'LoginGroup', 'wcfg':{'text':'Password'},
#                    'gridcfg':{'sticky':'e', 'row':2, 'column':1}
#                    })
#        ifd.append({'widgetType':Tkinter.Entry, 'name':'Password_Entry',
#                    'parent':'LoginGroup', 'wcfg':{'show':'*'},
#                    'gridcfg':{'sticky':'ew', 'row':2, 'column':2}
#                    })
#        ifd.append({'widgetType':Tkinter.Label, 'name':'Remember_Label',
#                    'parent':'LoginGroup',
#                    'wcfg':{'text':'Remember User Name and Password'},
#                    'gridcfg':{'sticky':'e', 'row':3, 'column':0,'columnspan':2}
#                    })
#        self.RememberLogin_var = Tkinter.BooleanVar()
#        ifd.append({'widgetType':Tkinter.Checkbutton, 'name':'Remember_Checkbutton',
#                    'parent':'LoginGroup', 'variable':self.RememberLogin_var,
#                    'gridcfg':{'sticky':'w', 'row':3, 'column':2}
#                    })       
        #AutoGrid group
        ifd.append({'name':"AutoGrid", 'widgetType':Pmw.Group,
                    'container':{'AutoGrid':'w.interior()'},
                    'wcfg':{'tag_text':'AutoGrid'},
                    'gridcfg':{'sticky':'nswe'}
                    })
        
        ifd.append({'widgetType':Tkinter.Button, 'name':'Run_autogrid',
                    'parent':'AutoGrid', 
                    'wcfg':{'text':'Run AutoGrid ',
                            'command':self.startAutogrid},
                    'gridcfg':{'sticky':'w', 'row':0, 'column':0}
                    })

        ifd.append( {'name': 'gpf_entry', 'parent':'AutoGrid',
                     'widgetType':Tkinter.Entry, 
                     'wcfg':{'width':30,'textvariable':self.gpf},
                     'gridcfg':{'sticky':'w','row':0,'column':1}
                     })

        ifd.append({'name': 'browse_gpf', 'widgetType': Tkinter.Button,
                    'parent':'AutoGrid', 'text':'Browse',
                    'command':self.browse_gpf,
                    'gridcfg':{'sticky':'w','row':0, 'column':2}
                    })
        #AutoDock group
        ifd.append({'name':"AutoDock", 'widgetType':Pmw.Group,
                    'container':{'AutoDock':'w.interior()'},
                    'wcfg':{'tag_text':'AutoDock'},
                    'gridcfg':{'sticky':'nswe'}
                    })

        ifd.append({'widgetType':Tkinter.Button, 'name':'Run_autodock',
                    'parent':'AutoDock', 
                    'wcfg':{'text':'Run AutoDock',
                            'command':self.startAutodock},
                    'gridcfg':{'sticky':'w', 'row':0, 'column':0}
                    })

        ifd.append( {'name': 'dpf_entry', 'parent':'AutoDock',
                     'widgetType':Tkinter.Entry, 
                     'wcfg':{'width':30,'textvariable':self.dpf},
                     'gridcfg':{'sticky':'w','row':0,'column':1}
                     })

        ifd.append({'name': 'browse_dpf', 'widgetType': Tkinter.Button,
                    'parent':'AutoDock', 'text':'Browse',
                    'command':self.browse_dpf,
                    'gridcfg':{'sticky':'w','row':0, 'column':2}
                    })

        ifd.append({'name': 'ag_local', 'widgetType': Tkinter.Radiobutton,
                    'parent':'AutoDock', 'text':'Use local grids',
                    'tooltip':"This option sends locally stored grid files with Web Services request",
                    'wcfg':{'variable':self.ad_radio,'value':0},
                    'gridcfg':{'sticky':'w','row':1, 'column':0,'columnspan':2}
                    })

#        ifd.append({'name': 'ag_before', 'widgetType': Tkinter.Radiobutton,
#                    'parent':'AutoDock', 'text':'Run AutoGrid first',
#                    'tooltip':"This option runs AutoGrid Web Services and uses resulting map files for AutoDock",
#                    'wcfg':{'variable':self.ad_radio,'value':1,'state':'disabled'},
#                    'gridcfg':{'sticky':'w','row':2, 'column':0,'columnspan':2}
#                    })

        ifd.append({'name': 'use_remote', 'widgetType': Tkinter.Radiobutton,
                    'parent':'AutoDock', 'text':'Use grids from server directory',
                    'tooltip':"This option copies map files from previous AutoGrid run",
                    'wcfg':{'variable':self.ad_radio,'value':2,},
                    'gridcfg':{'sticky':'w','row':3, 'column':0,'columnspan':2}
                    })

        ifd.append( {'name': 'remote_dir', 'parent':'AutoDock',
                     'widgetType':Tkinter.Entry, 
                     'wcfg':{'width':23,'textvariable':self.prev_dir},
                     'gridcfg':{'sticky':'e','row':3,'column':1,'columnspan':2}
                     })

        #Status
        ifd.append({'name':"StatusGroup", 'widgetType':Pmw.Group,
                    'container':{'StatusGroup':'w.interior()'},
                    'wcfg':{'tag_text':'Web Services Status'},
                    'gridcfg':{'sticky':'nswe'}
                    })
        
        ifd.append({'widgetType':Tkinter.Label, 'name':'status0',
                    'parent':'StatusGroup', 
                    'wcfg':{'text':'   ',},
                    'gridcfg':{'sticky':'w', 'row':0, 'column':0}
                    })

        ifd.append({'widgetType':Tkinter.Label, 'name':'status1',
                    'parent':'StatusGroup', 
                    'wcfg':{'text':'   ',},
                    'gridcfg':{'sticky':'w', 'row':1, 'column':0}
                    })

        ifd.append({'name':'WS_ProgressBar', 'widgetType':Tkinter.Frame,
                    'parent':'StatusGroup', 'wcfg':{'height':30},
                     'gridcfg':{'sticky':'ew', 'row':2,'column':0}
                    })
        
        ifd.append({'widgetType':Tkinter.Label, 'name':'down_label',
                    'parent':'StatusGroup', 
                    'wcfg':{'text':'   ',},
                    'gridcfg':{'sticky':'w', 'row':3, 'column':0}
                    })
        
        return ifd
    
    def browse_gpf(self):
        filename = askopenfilename(filetypes=[('Grid Parameter File','*.gpf')],\
                                   title="Please Select Grid Parameter File",
                                   parent=self.cmdForms['default'].root)
        if filename:
            self.gpf.set(filename)
            self.cmdForms['default'].descr.entryByName['Run_autogrid']['widget'].configure(state='normal')
            #self.cmdForms['default'].descr.entryByName['ag_before']['widget'].configure(state='normal')

    def browse_dpf(self):
        filename = askopenfilename(filetypes=[('Dock Parameter File','*.dpf')],\
                                   title="Please Select Dock Parameter File",
                                   parent=self.cmdForms['default'].root)
        if filename:
            self.dpf.set(filename)
            self.cmdForms['default'].descr.entryByName['Run_autodock']['widget'].configure(state='normal')
           
    def initForm(self, cmdForm=None):
        cmdForm.descr.entryByName['WS_address']['widget'].selectitem(0)
#        if not os.path.exists(self.rc_ad):
#            open(self.rc_ad,'w')
#        else:
#            file = open(self.rc_ad)
#            text = file.read()
#            text = text.split()
#            for line in text:
#                tmp_line = line.split('User:')
#                if len(tmp_line) > 1:
#                    cmdForm.descr.entryByName['UserName_Entry']['wcfg']\
#                    ['textvariable'].set(tmp_line[1])
#                tmp_line = line.split('Password:')
#                if len(tmp_line) > 1:
#                    cmdForm.descr.entryByName['Password_Entry']['wcfg']\
#                    ['textvariable'].set(tmp_line[1])
#            file.close()
            
#        def openurl(event):
#            webbrowser.open('https://nbcr.net:8443/worksphere/start?cid=apply')
#        cmdForm.descr.entryByName['New_User']['widget'].bind(sequence="<Button-1>",
#                                                                   func=openurl)
        if hasattr(self.vf,'dpo') and self.vf.dpo.dpf_filename:
            self.dpf.set(self.vf.dpo.dpf_filename)
            cmdForm.descr.entryByName['Run_autodock']['widget'].configure(state='normal')
        else:
            if not self.dpf.get():
                cmdForm.descr.entryByName['Run_autodock']['widget'].configure(state='disabled')

        if hasattr(self.vf,'gpo') and self.vf.gpo.gpf_filename:
            self.gpf.set(self.vf.gpo.gpf_filename)
            cmdForm.descr.entryByName['Run_autogrid']['widget'].configure(state='normal')
            #cmdForm.descr.entryByName['ag_before']['widget'].configure(state='normal')
        else:
            if not self.gpf.get():
                cmdForm.descr.entryByName['Run_autogrid']['widget'].configure(state='disabled')

        self.progressBar = ProgressBar(
                          cmdForm.descr.entryByName['WS_ProgressBar']['widget'], 
                          labelside=None, width=200, height=20, mode='percent')

        self.progressBar.setLabelText('Progress...')
        self.progressBar.set(0)
        cmdForm.descr.entryByName['WS_ProgressBar']['widget'].grid_forget()        

    def startAutogrid(self):
        self.cmdForms['default'].descr.entryByName['Run_autogrid']['widget']\
                                                    .configure(state='disabled')
        gpf_file = self.gpf.get()
        if not os.path.exists(gpf_file):
            self.cmdForms['default'].descr.entryByName['status0']['widget'].\
            configure(text = 'ERROR: gpf file ' + gpf_file + ' does not exist!')
            return
        
        self.host = self.cmdForms['default'].descr.entryByName['WS_address']['widget'].get()
        
#        if not self.login :
#            self.cmdForms['default'].descr.entryByName['status0']['widget'].\
#                configure(text='Connecting to '+ self.host + ". Please wait...")
#            self.vf.GUI.ROOT.update()
#            f = self.validate_login()
#            if f == "Failed":
#                return
        
        self.appLocator = AppServiceLocator()
        self.req = launchJobRequest()
        input_file = os.path.basename(gpf_file)
        options = '-p ' +  input_file + ' -l ' + os.path.splitext(input_file)[0] + '.glg'
        self.req._argList = options
        
        #input_gpf = ns0.InputFileType_Def('inputFile')
        #input_gpf._name = input_file
        gpfFile = open(gpf_file, 'r')
        gpfFileString = gpfFile.read()
        gpfFile.close()
        #input_gpf._contents = gpfFileString
        
        gpfFileString = gpfFileString.split('\n')
        for line in gpfFileString:
            if line[0:9] == 'receptor ':
                pdbqs = line.split()[1]
        
        #input_pdbqs = ns0.InputFileType_Def('inputFile')
        #input_pdbqs._name = pdbqs
        pdbqs = os.path.join(os.path.split(gpf_file)[0],pdbqs)
        #pdbqsFile = open(pdbqs, 'r')
        #pdbqsFileString = pdbqsFile.read()
        #pdbqsFile.close()
        #input_pdbqs._contents = pdbqsFileString
        
        inputFiles = []
        
        #inputFiles.append(input_gpf)
        #inputFiles.append(input_pdbqs)
        inputFiles.append(self.uploadFile(gpf_file))
        inputFiles.append(self.uploadFile(pdbqs))
        
        self.req._inputFile = inputFiles
        
        self.appServicePort = self.appLocator.getAppServicePort(
                                 self.host+'/AutogridOpalService')
        
        resp = self.appServicePort.launchJob(self.req)

        self.JobID = resp._jobID
        self.cmdForms['default'].descr.entryByName['status0']['widget'].\
                                configure(text = 'Running Autogrid Job ID: ' + self.JobID)
        self.vf.GUI.ROOT.update()
        self.vf.GUI.ROOT.after(5, self.checkStatus)
        self.cmdForms['default'].descr.entryByName['Run_autogrid']['widget'].configure(state='normal')
        self.prev_dir.set(self.JobID)
        self.cmdForms['default'].descr.entryByName['use_remote']['widget'].configure(state='normal')
        
    def startAutodock(self):

        self.cmdForms['default'].descr.entryByName['Run_autodock']['widget']\
                                                    .configure(state='disabled')
        dpf_file = self.dpf.get()
        if not os.path.exists(dpf_file):
            self.cmdForms['default'].descr.entryByName['status0']['widget'].\
            configure(text = 'ERROR: dpf file ' + fpf_file + ' does not exist!')
            return
        
        self.host = self.cmdForms['default'].descr.entryByName['WS_address']['widget'].get()
        
#        if not self.login :
#            self.cmdForms['default'].descr.entryByName['status0']['widget'].\
#                configure(text='Connecting to '+ self.host + ". Please wait...")
#            self.vf.GUI.ROOT.update()
#            f = self.validate_login()
#            if f == "Failed":
#                return

        self.appLocator = AppServiceLocator()
        self.req = launchJobRequest()
        input_file = os.path.basename(dpf_file)
        options = '-p ' +  input_file + ' -l ' + os.path.splitext(input_file)[0] + '.dlg'
        self.req._argList = options
        
        #input_dpf = ns0.InputFileType_Def('inputFile')
        #input_dpf._name = input_file
        dpfFile = open(dpf_file, 'r')
        dpfFileString = dpfFile.read()
        dpfFile.close()
        #input_dpf._contents = dpfFileString
        #DPF file
        inputFiles = []       
        inputFiles.append(self.uploadFile(dpf_file))
        
        
        run_option = self.ad_radio.get()
        if run_option == 0: # sends locally stored grid files
            inputs = re.findall("\w*.\w*\.map ",dpfFileString)
            inputs.extend(re.findall("\w*\.maps.fld",dpfFileString))
            inputs.extend(re.findall("\w*.pdbq[t]*",dpfFileString))
            for input in inputs:
                input = input.strip()
                #ws_input = ns0.InputFileType_Def('inputFile')
                #ws_input._name = input
                input_full_name = os.path.join(os.path.split(dpf_file)[0],input)
                #inputFile = open(input_full_name, 'r')
                #inputFileString = inputFile.read()
                #inputFile.close()
                #ws_input._contents = inputFileString
                inputFiles.append(self.uploadFile(input_full_name))
                
        elif run_option == 2: # runs AutoGrid first
            prev_dir = self.prev_dir.get()
            inputs = re.findall("\w*.\w*\.map ",dpfFileString)
            inputs.extend(re.findall("\w*\.maps.fld",dpfFileString))
            
            host = 'http://'+self.host.split('/')[2]
            for input in inputs:
                self.req._argList += " " +host+"/"+prev_dir+"/"+input
            pdbq_input = re.findall("\w*.pdbq[t]*",dpfFileString)
            pdbq_input = pdbq_input[0].strip()

            #ws_input = ns0.InputFileType_Def('inputFile')
            #ws_input._name = pdbq_input
            input_full_name = os.path.join(os.path.split(dpf_file)[0],pdbq_input)
            #inputFile = open(input_full_name, 'r')
            #inputFileString = inputFile.read()
            #inputFile.close()
            #ws_input._contents = inputFileString
            inputFiles.append(self.uploadFile(input_full_name))
        self.req._inputFile = inputFiles
        self.vf.GUI.ROOT.update()
        self.appServicePort = self.appLocator.getAppServicePort(
                                 self.host+'/AutodockOpalService',)
        resp = self.appServicePort.launchJob(self.req)

        self.JobID = resp._jobID
        self.cmdForms['default'].descr.entryByName['status0']['widget'].\
                                configure(text = 'Running Autodock Job ID: ' + self.JobID)
        self.vf.GUI.ROOT.update()
        self.vf.GUI.ROOT.after(5, self.checkStatus)
        self.cmdForms['default'].descr.entryByName['Run_autodock']['widget'].configure(state='normal')

    def uploadFile(self, path):
        """
        this function given a string containing a path creates a 
        InputFileType to be used with jobLaunch
        """
        inputFile = ns0.InputFileType_Def('inputFile')
        inputFile._name = os.path.basename(path)
        if self.isOpal2():
            #use attachment this is opal2 server
            inputFile._attachment = open(path, "r")
        else:
            #it's not a opal2 server don't user attachment
            infile = open(path, "r")
            inputFile._contents = infile.read()
            infile.close()
        return inputFile

        
    def isOpal2(self):
        """return True if we are using Opal2"""
        print "self.host is: " + self.host
        if self.host.find("/opal2/") != -1:
            return True
        else:
            return False

    def checkStatus(self):
        resp = self.appServicePort.queryStatus(queryStatusRequest(self.JobID))
        if resp._code == 8: # 8 = GramJob.STATUS_DONE
            descr = self.cmdForms['default'].descr
            descr.entryByName['status0']['widget'].configure(text=resp._message)       
            webbrowser.open(resp._baseURL)
            descr.entryByName['status1']['widget'].configure(text=resp._baseURL,
                                                       fg='Blue',cursor='hand1')
            def openurl(event):
                webbrowser.open(resp._baseURL)
            descr.entryByName['status1']['widget'].bind(sequence="<Button-1>",
                                                                   func=openurl)
            self.resp = self.appServicePort.getOutputs(getOutputsRequest(self.JobID))
            descr.entryByName['WS_ProgressBar']['widget'].grid(sticky='ew', 
                                                                row=2, column=0)

            self.opener = urllib.FancyURLopener(cert_file=self.proxy_gama, 
                                                       key_file=self.proxy_gama)
            self.download_finished = False
            self.new_download = True
            self.file_counter = -1
            inputs = [x for x in self.resp._outputFile if x._name[-3:] !='dlg']
            if len(inputs) != len(self.resp._outputFile):
                for input in inputs:
                    self.resp._outputFile.remove(input)
            self.download()                
            return
        else:
            self.cmdForms['default'].descr.entryByName['status0']['widget'].\
                                    configure(text = "Status: " + resp._message)       
            self.cmdForms['default'].descr.entryByName['status1']['widget'].\
                                    configure(text = "")
            
        self.vf.GUI.ROOT.after(5000, self.checkStatus)

    def download(self):
        if self.new_download:
            self.file_counter += 1
            if self.file_counter > self.resp._outputFile.__len__() - 1 :
                self.cmdForms['default'].descr.entryByName['WS_ProgressBar']\
                                                        ['widget'].grid_forget()
                self.cmdForms['default'].descr.entryByName['down_label']\
                                                 ['widget'].configure(text = "")
                self.cmdForms['default'].descr.entryByName['Run_autogrid']\
                                            ['widget'].configure(state='normal')
                return
            
            self.progressBar.configure(progressformat='percent',
                                        labeltext='Progress ... ', max =100)                   
            self.progressBar.set(0)
            remote_file = self.resp._outputFile[self.file_counter]
            
            self.cmdForms['default'].descr.entryByName['down_label']['widget'].\
                configure(text = "Downloading " + remote_file._name + "      " +
                                              str(self.file_counter+1) +" of " + 
                                           str(self.resp._outputFile.__len__()))

            self._url = self.opener.open(remote_file._url)
            try:
                self._out = open(remote_file._name,"w")
            except IOError:
                showerror("Download Failed!", 
                          "Permission denied: " +os.path.join(os.getcwd(),remote_file._name),
                          parent = self.cmdForms['default'].root)
                return             
            bytes = int(self._url.headers.dict['content-length'])
            self._progress_counter = 0
            self._download_bytes = bytes/100
            if self._download_bytes == 0: self._download_bytes = 1
            self.new_download = False
            self.vf.GUI.ROOT.after(1, self.download)
            return
        else:
            self._progress_counter += 1
            if self._progress_counter >  100:
                self._progress_counter =  100
            self.progressBar.set(self._progress_counter)
            tmp = self._url.read(self._download_bytes)
            if tmp:
                self._out.write(tmp)
            else:
                self._url.close()
                self._out.close()
                self.new_download = True
                
            self.vf.GUI.ROOT.after(50, self.download)
        
    def validate_login(self):
        self.login = False
        from mglutil.web.services.SecuritymyproxyloginImplService_services import \
                                          loginUserMyProxyRequestWrapper, \
                                      SecuritymyproxyloginImplServiceLocator
        gamaLoginLocator = SecuritymyproxyloginImplServiceLocator()
        gamaLoginService = gamaLoginLocator.getSecuritymyproxyloginImpl(
                                    ssl=1,transport=httplib.HTTPSConnection)
        req = loginUserMyProxyRequestWrapper()
        username =  self.cmdForms['default'].descr.\
                               entryByName['UserName_Entry']['widget'].get()
        passwd =  self.cmdForms['default'].descr.\
                               entryByName['Password_Entry']['widget'].get()
        if not username or not passwd:
            showerror("Username or Password is missing", 
                      "Login failed. Please type your User Name and Password,\
 or click on New User?", parent = self.cmdForms['default'].root)
            return "Failed"
        req._username = username
        req._passwd = passwd
        resp = gamaLoginService.loginUserMyProxy(req)
        f = open(self.proxy_gama, "w")
        f.write(resp._loginUserMyProxyReturn)
        f.close()
        if self.RememberLogin_var.get():
            file = open(self.rc_ad,'w')
            user = self.cmdForms['default'].descr.entryByName\
                                          ['UserName_Entry']['widget'].get()
            passwd = self.cmdForms['default'].descr.entryByName\
                                          ['Password_Entry']['widget'].get()
            file.write("User:%s\nPassword:%s\n"%(user,passwd))
        self.login = True
        
WebServicesGUI=CommandGUI()
WebServicesGUI.addMenuCommand('AutoToolsBar', menuText['StartMB'], "Web Services...")

commandList = [{'name':'ADweb_services','cmd':WebServices(),'gui':WebServicesGUI}]  

WebServices4GUI=CommandGUI()
WebServices4GUI.addMenuCommand('AutoTools4Bar', menuText['StartMB'], "Web Services...")


def initModule(viewer):
    if not hasattr(viewer, 'ADweb_services') and hasattr(viewer, 'GUI')\
        and hasattr(viewer.GUI, 'currentADTBar'):
        viewer.addCommand(WebServices(),'ADweb_services',WebServices4GUI)
    #else:
    #    for _dict in commandList:
    #        viewer.addCommand(_dict['cmd'],_dict['name'],_dict['gui'])
